      implicit double precision ( a - h, o - y )
      implicit double complex ( z )
      parameter( mx = 20, my = 1000, mxy = mx * my )
      parameter( pi = 3.1415926535898d0 )
      dimension zg( mxy ), zz( mxy ), zd( mxy ), ad( mxy )
      dimension ixmin( my ), zmin( my ), num( 2 )

c     This function finds contours which zigzag left and right 
c     around zeros of the zeta 
c     function in the strip 
c     x0-xzig<Re s<x0+xzig, boxstart<Im s<boxstart + boxheight, 
c     and then computes
c     (1/2*pi*i) times the integral of the log derivative of zeta
c     along the contours.
c     Output:
c     fort.10	gx	Grid point x-coordinate 1:nz
c     fort.11	gy	Grid point y-coordinate 1:nz
c     fort.12	ad	Absolute value of log derivative at grid pt 1:nz
c     fort.13	px	Path x 1:ny
c     fort.14	py	Path y 1:ny
c     fort.15	lb	Lower bound 1:ny
c     fort.16	ub	Upper bound 1:ny

      cc = -40.0d0
      nl = 5
      np = 2
      nx = 5
      x0 = -0.5d0
      xzig = 0.1d0
      boxstart = 260.0d0
      boxheight = 10.0d0
      nbox = 1

      write( 6, *) ' enter cc, nl, np, nx, x0, xzig'
      read( 5, *, err=2000, end=2000) cc, nl, np, nx, x0, xzig
      write( 6, *) ' cc, nl, np, nx, x0, xzig = ',
     >      cc, nl, np, nx, x0, xzig
      
      write( 6, * ) ' boxstart, boxheight, nbox '
      read( 5, *, err=2000, end=2000 ) boxstart, boxheight, nbox
      write( 6, * ) ' boxstart, boxheight, nbox = ',
     >      boxstart, boxheight, nbox

      iogx = 1
      iogy = 1
      ioad = 1
      iopx = 1
      iopy = 1
      iolb = 1
      ioub = 1

      write( 6, * ) ' iogx, iogy, ioad, iopx, iopy, iolb, ioub '
      read( 5, *, err=2000, end=2000 ) 
     >      iogx, iogy, ioad, iopx, iopy, iolb, ioub
      write( 6, * ) ' iogx, iogy, ioad, iopx, iopy, iolb, ioub = ',
     >      iogx, iogy, ioad, iopx, iopy, iolb, ioub

c     The factor 1.5 ensures that left and right contours do not cross.
      xleft = x0 - 1.5d0 * xzig
      xright = x0 + 1.5d0 * xzig

1000  format( 1(1x, g18.12) )

      do ibox = 1, nbox

c     In each box, we zigzag left (1) and right (2) to get 
c     upper and lower bounds.
      do ixbox = 1, 2
      if( ixbox .eq. 1 ) then
         x1 = xright - xzig
         x2 = xright + xzig
      else
         x1 = xleft - xzig
         x2 = xleft + xzig
      end if
      hx = ( x2 - x1 ) / max( 1, nx - 1 )
 
      y1 = boxstart + ( ibox - 1) * boxheight
      y2 = y1 + boxheight
      ny = max( 2, nint( ( y2 - y1 ) / hx ) )
      hy = ( y2 - y1 ) / max( 1, ny - 1 )

      nz = nx * ny
      do iy = 1, ny
         yy = y1 + ( iy - 1 ) * hy
         do ix = 1, nx
            xx = x1 + ( ix - 1 ) * hx
            zg( ( iy - 1 ) * nx + ix ) = dcmplx( xx, yy )
         end do
      end do
      if( iogx .ne. 0 ) then
         write( 10, 1000 ) ( 
     >     dble( zg( iz ) ), 
     >      iz = 1, nz )
      end if
      if( iogy .ne. 0 ) then
         write( 11, 1000 ) ( 
     >      imag( zg( iz ) ), 
     >      iz = 1, nz )
      end if

      call zeta( nz, zg, zz, zd, cc, nl, np )

      do ixy = 1, nz
        ad( ixy ) = abs( zd( ixy ) )
      end do
      if( ioad .ne. 0 ) then
         write( 12, 1000 ) ( ad( iz ), iz = 1, nz )
      end if

      do iy = 1, ny
        ixbeg = ( iy - 1 ) * nx

        ixm = 1
        tmin = ad( ixbeg + 1 )
        do ix = 2, nx
           adix = ad( ixbeg + ix )
           if( adix .lt. tmin ) then
              tmin = adix
              ixm = ix
           end if
        end do
        ixmin( iy ) = ixm
      end do

      if( iopx .ne. 0 ) then
         write( 13, 1000 ) ( 
     >    dble( zg( ( iy - 1 ) * nx + ixmin( iy ) ) ), 
     >    iy = 1, ny )
      end if

      if( iopy .ne. 0 ) then
         write( 14, 1000 ) ( 
     >    imag( zg( ( iy - 1 ) * nx + ixmin( iy ) ) ), 
     >    iy = 1, ny )
      end if

      zsum = 0.0d0
      do iy = 1, ny - 1
        zvy = zz( ( iy - 1 ) * nx + ixmin( iy ) )
        ip = iy + 1
        zvp = zz( ( ip - 1 ) * nx + ixmin( ip ) )
        zterm = log( zvp / zvy )
        zsum = zsum + zterm
      end do
      zsum = 0.50d0 * dcmplx( 0.0d0, 1.0d0 ) * zsum / pi 
      num( ixbox ) = nint( dble( zsum ) )

      end do
      
      write( 6, * ) ' y1, y2, num = ', y1, y2, num( 1 ), num( 2 )
      if( iolb .ne. 0 ) then
         write(15, *) num( 1 )
      end if
      if( ioub .ne. 0 ) then
         write(16, *) num( 2 )
      end if

      end do

2000  continue

      end
c        1         2         3         4         5         6          72
      subroutine zeta( nz, zs, zz, zd, cc, nl, np )
c     Evaluate vector of values zz = zeta( zs, c ) and log derivatives 
c     zd = zeta'/zeta where c is parameter
c     in map, s is vector of powers in zeta sum, nl is number of levels
c     and np number of basis functions. Should give close to zero for s 
c     the dimension associated with c: 
c     zeta( 1, 0.31850809575800, z, -20, 4, 1 ) 
c       gives z = 2.7859e-10 +i* -1.0229e-22
c     and
c     zeta( 1, 0.48479829443816, -5, z, 4, 1 )
c       gives z = 1.2824e-06 +i* -1.1619e-20
c     while with higher accuracy
c     zeta( 1, 0.31850809575800, -20, z, 8, 5 )
c     	gives z = -1.1571e-14 +i* -3.3773e-17
c     and
c     zeta( 1, 0.48479829443816, -5, z, 8, 5 )
c    	gives z = -6.5302e-16 +i* -2.7861e-17
c     

      implicit double precision (a-h,o-y)
      implicit double complex (z)
      parameter( pi = 3.1415926535898d0 )

c     Max number of levels, intervals and basis functions per interval.
      parameter( ml = 8, mi = 2**ml, mp = 5, mip = mi * mp )

c     Order of accuracy for numerical quadrature and number of pts/wts.
c     Tenth order quadrature formula on unit disk from AS p. 892.
      parameter( nq = 21 )

c     Quadrature points and weights.
      dimension zq( nq ), wq( nq )

      data zq /
     > ( 0.0d0, 0.0d0 ),
     > ( 0.4820621466885461, 0.3502386507107874 ),
     > ( 0.1841313553452870, 0.5666980410239565 ),
     > ( -0.1841313553452870, 0.5666980410239566 ),
     > ( -0.4820621466885460, 0.3502386507107875 ),
     > ( -0.5958615826865181, 0.0d0 ),
     > ( -0.4820621466885461, -0.3502386507107874 ),
     > ( -0.1841313553452871, -0.5666980410239565 ),
     > ( 0.1841313553452869, -0.5666980410239566 ),
     > ( 0.4820621466885460, -0.3502386507107875 ),
     > ( 0.5958615826865181, 0.0d0 ),
     > ( 0.7436573695963749, 0.5402987052763671 ),
     > ( 0.2840518392014725, 0.8742216692147242 ),
     > ( -0.2840518392014724, 0.8742216692147243 ),
     > ( -0.7436573695963747, 0.5402987052763673 ),
     > ( -0.9192110607898046, 0.0d0 ),
     > ( -0.7436573695963749, -0.5402987052763670 ),
     > ( -0.2840518392014726, -0.8742216692147242 ),
     > ( 0.2840518392014723, -0.8742216692147243 ),
     > ( 0.7436573695963747, -0.5402987052763674 ),
     > ( 0.9192110607898046, 0.0d0 ) 
     > /
      data wq /
     > 0.3490658503988659,
     > 10 * 0.1610021706622441,
     > 10 * 0.1182505096568486 
     > /

c     Input/output points and values---must be at least nz long.
      dimension zs( * ), zz( * ), zd( * )

c     Interval endpoints and indices.
      dimension emin( mi ), emax( mi ), indices( mi, ml )
      dimension tmp( 2 * mi ), itmp( mi, ml )

c     Matrix of operator and small block matrix.
      dimension zma( mip, mip ), zta( mp, mp )
      dimension zmd( mip, mip ), ztd( mp, mp )
      dimension ipiv( mip )
c     Temporary arrays.
      dimension center( mi ), radius( mi ), sg( ml )
      dimension zqi( nq ), zik( mp ), zbjl( mp ), sqi( mp )

c     Integer matrix to show block structure.
      dimension istruct( mi, mi )

c     Test input parameters so arrays don't overflow.
      if( nl .gt. ml ) then
         write( 6, * ) ' nl = ', nl, ' > ml = ', ml, ' error return'
      end if
      if( np .gt. mp ) then
         write( 6, * ) ' np = ', np, ' > mp = ', mp, ' error return'
      end if
c     Here we should build a quadrature formula that gives desired 
c     accuracy depending on the physical parameters c and s. In the 
c     meantime, we use tenth order quadrature formula on unit disk 
c     from AS p. 892.
c      zq( 1 ) = dcmplx( 0.0d0, 0.0d0 )
c      wq( 1 ) = pi * 1.0d0 / 9.0d0
c      ibeg = 1
c      ss = dsqrt( 0.10d0 * ( 6.0d0 - dsqrt( 6.0d0 ) ) )
c      ww = pi * ( 16.00d0 + dsqrt( 6.00d0 ) ) / 360.0d0
c      do iq = 1, 10
c         tq = 0.20d0 * pi * dble( iq )
c         zq( ibeg + iq ) = ss * dcmplx( cos( tq ), sin( tq ) )
c         wq( ibeg + iq ) = ww
c      end do
c
c      ibeg = 11
c      ss = dsqrt( 0.10d0 * ( 6.0d0 + dsqrt( 6.0d0 ) ) )
c      ww = pi * ( 16.0d0 - dsqrt( 6.0d0 ) ) / 360.0d0
c      do iq = 1, 10
c         tq = 0.20d0 * pi * dble( iq )
c         zq( ibeg + iq ) = ss * dcmplx( cos( tq ), sin( tq ) )
c         wq( ibeg + iq ) = ww
c      end do
c
c     Initialize and set arrays to maximum sizes required.
 
      ni = 2
      dd = 0.50d0 * ( 1.00d0 + dsqrt( 1.0d0 - 4.00d0 * cc ) )

      e1 = dsqrt( -cc - dd ) - 0.250d0
      e2 = -dd - 0.250d0
      emin( 1 ) = min( e1, e2 )
      emin( 2 ) = max( e1, e2 )
      e1 = dd + 0.250d0
      e2 = -dsqrt(-cc - dd ) + 0.250d0
      emax( 1 ) = min( e1, e2 )
      emax( 2 ) = max( e1, e2 )
      indices( 1, 1 ) = 1
      indices( 2, 1 ) = 2
 
c     Iterate through levels il, mapping and sorting interval endpts.
      do il = 2, nl
  
         ni2 = 2 * ni

         do ii = 1, ni
            ji = ii + ni
            tmp( ii ) = -1.0d0 * dsqrt( emin( ii ) - cc )
            tmp( ji ) = dsqrt( emin( ii ) - cc )
         end do
         do ii = 1, ni2
            emin( ii ) = tmp( ii )
         end do
 
         do ii = 1, ni
            ji = ii + ni
            tmp( ii ) = -1.0d0 * dsqrt( emax( ii ) - cc )
            tmp( ji ) = dsqrt( emax( ii ) - cc )
         end do
         do ii = 1, ni2 
            emax( ii ) = tmp( ii )
         end do
    
         do jl = 1, il - 1
            do ii = 1, ni
               ji = ii + ni
               itmp( ii, jl + 1 ) = indices( ii, jl )
               itmp( ji, jl + 1 ) = indices( ii, jl )
            end do
         end do
         do ii = 1, ni
            ji = ii + ni
            itmp( ii, 1 ) = 1
            itmp( ji, 1 ) = 2
         end do
         do jl = 1, il
            do ii = 1, ni2
               indices( ii, jl ) = itmp( ii, jl )
            end do
         end do

         ni = ni2
         nip = ni * np

c        Reorder emin and emax as necessary so emin(ii) < emax(ii) foreach ii.
         do ii = 1, ni
           if( emin( ii ) .gt. emax( ii ) ) then
             swap = emax( ii )
             emax( ii ) = emin( ii )
             emin( ii ) = swap
           end if
         end do

c        End loop through levels do il = 2, nl
      end do

c     Now we have the endpts in pairs: form matrix of transfer operator
c     in basis of complex powers centered and scaled to each interval.

      do ii = 1, ni
         radius( ii ) = 0.50d0 * ( emax( ii ) - emin( ii ) )
         center( ii ) = emin( ii ) + radius( ii )
      end do

c     Loop through input values of s, evaluating zeta and zd at each point zt.
      do iz = 1, nz
         zt = zs( iz )

c        Initialize big matrices zma and zmd to complex zeroes.
         nip = ni * np
         do i = 1, nip
            do j = 1, nip
               zma( i, j ) = dcmplx( 0.0d0, 0.0d0 )
               zmd( i, j ) = dcmplx( 0.0d0, 0.0d0 )
            end do
         end do
         do ii = 1, ni
            do ji = 1, ni
               istruct( ii, ji ) = 0
            end do
         end do
  
c        Loop through rows ii, building two nonzero blocks per row.
         do ii = 1, ni

c           Each nonzero block in column ji corresponds to ib =1 or 2.
            do ib = 1, 2

c              Compute index ji of nonzero block for index ib.
               sg( 1 ) = ib
               do il = 2, nl
                  sg( il )  = indices( ii, il - 1 )
               end do
               ji = 1
               m = 1
               do il = nl, 1, -1
                  ji = ji + ( sg( il ) - 1 ) * m
                  m = 2 * m
               end do

               istruct( ii, ji ) = ib

c              Form np x np block (ii, ji) by integrating basis functions against 
c              transformed basis functions times power of square root over circle 
c              D_ii.  
c              a = block( ii, ji, ib, np, nl, center, radius, indices, cc, zt )
c              Build nonzero PxP block at location ( ii, ji ) in block matrix, by 
c              integrating f_ik* vs L_b f_jl over circle D_ii for basis functions 
c              1 <= k,l <= np on circles:
c        
c              f_ik(z) = ( 1/r_ii) dsqrt( (k+1) / pi ) ( ( z - c_ii ) / r_ii )**k
c        
c              Here f_jl is transformed by L_ib to yield
c           
c              L_b f_jl(z) = [ g_ib'(z) ]**zt f_jl( g_ib(z) ).
c           
c              The square brackets indicate that we always take the positive sign so 
c           
c              [ g_b'(z) ] = 1 / ( 2 * dsqrt( z - cc ) ).
        
               ri = radius( ii )
               ci = center( ii )
               rj = radius( ji )
               cj = center( ji )
     
c              Shift and scale quadrature points to D_i
               do iq = 1, nq
                  zqi( iq ) = ci + ri * zq( iq )
               end do
       
c              Integrate f_ik* vs L_b f_jl over D_i and accumulate in zta( ip, jp )
      
c              Initialize to zeroes first.
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = dcmplx( 0.0d0, 0.0d0 )
                     ztd( ip, jp ) = dcmplx( 0.0d0, 0.0d0 )
                  end do
               end do
       
               do iq = 1, nq
                  z = zqi( iq )
                  w = wq( iq )
       
c                 This wastes arithmetic since we are undoing what we just did.
c                 zi = dconjg( z - ci ) / ri
c                 This does less arithmetic and produces different results.
                  zi = dconjg( zq( iq ) )
       
c                 Matlab code from zlogd.m
c                 gs = (4*( z - c ))^(-s/2);
c                 wgs = w * gs;
c                 % For the derivative wrt s to be stored in ztd.
c                 wlg = -0.5 * wgs * log(4*( z - c ));
 
                  zgs = ( 4.0d0 * ( z - cc ) )**( -0.50d0 * zt )
                  zwgs = w * zgs
                  zwlg = -0.50d0 * zwgs * log( 4.0d0 * ( z - cc ) );
       
                  zb =(-1)**ib * sqrt( z - cc )
                  zbj = dconjg( zb - cj ) / rj

c                 Tabulate powers. 
c                 Later we should just do this once for all blocks to save time.
                  zik( 1 ) = dcmplx( 1.0d0, 0.0d0 )
                  do k = 1, np - 1
                     zik( k + 1 )  = zik( k ) * zi
                  end do
    
                  zbjl( 1 ) = dcmplx( 1.0d0, 0.0d0 )
                  do k = 1, np - 1
                     zbjl( k + 1 )  = zbjl( k ) * zbj
                  end do
       
c                 a = a + wgs * zik * zbjl'
c                 d = d + wlg * zik * zbjl';

                  do ip = 1, np
                     do jp = 1, np
                        zterm = zik( ip ) * dconjg( zbjl( jp ) )
                        zta( ip, jp ) = zta( ip, jp ) + zwgs * zterm
                        ztd( ip, jp ) = ztd( ip, jp ) + zwlg * zterm
                     end do
                  end do

c                 End of do iq = 1, nq over quadrature points.
               end do

c              Final scaling by factors that don't depend on anything else.
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = ri * zta( ip, jp ) / ( rj * pi )
                     ztd( ip, jp ) = ri * ztd( ip, jp ) / ( rj * pi )
                  end do
               end do
     
               do ip = 1, np
                  sqi( ip ) = dsqrt( dble( ip ) )
               end do
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = sqi( ip ) * zta( ip, jp )
                     ztd( ip, jp ) = sqi( ip ) * ztd( ip, jp )
                  end do
               end do
               do jp = 1, np
                  do ip = 1, np
                     zta( ip, jp ) = sqi( jp ) * zta( ip, jp )
                     ztd( ip, jp ) = sqi( jp ) * ztd( ip, jp )
                  end do
               end do
    
c              Store blocks in zma and zmd in location ( ii, ji ).
c Lm( ( ii - 1 ) * np + 1 : ii * np, ( ji - 1 ) * np + 1 : ji * np ) = a
    
               do ip = 1, np
                  iiip = ( ii - 1 ) * np + ip
                  do jp = 1, np
                     jijp = ( ji - 1 ) * np + jp
                     zma( iiip, jijp ) = -zta( ip, jp )
                     zmd( iiip, jijp ) = -ztd( ip, jp )
                  end do
               end do

c              End do ib = 1, 2 loop through signs of g operator.
            end do
c           End do ii = 1, ni loop through blocks of big matrix.
         end do

c        Replace zma by I + zma --- note have already changed sign in zma above.
         do i = 1, nip
            zma( i, i ) = dcmplx( 1.0d0, 0.0d0 ) + zma( i, i )
         end do
 
         if( nip .le. 8 ) then
c           Print matrices.

            write( 6, * ) ' initial zma = '
            do i = 1, nip
               write( 6, * ) ( zma( i, j ), j = 1, nip )
            end do

            write( 6, * ) ' initial zmd = '
            do i = 1, nip
               write( 6, * ) ( zmd( i, j ), j = 1, nip )
            end do

         end if

c        Calculate zeta function det( I - Lm ).
c        zz( iz ) = det( zma )
c        Calculate logarithmic derivative of zeta function.
c        zd( iz ) = trace( zma \ zmd )

c        Overwrite zma with its PLU factors (U on diagonal) and zmd with
c        zma^-1 * zmd, modulo permutation P stored in ipiv.
         call zgesv( nip, nip, zma, mip, ipiv, zmd, mip, info )

c        Extract sign of determinant from permutation: row i swapped with ipiv(i).
         idet = 1
         do i = 1, nip
            if( ipiv( i ) .ne. i ) idet = -idet
         end do
c        Extract determinant from U factor (ignore permutation).
         zdet = dcmplx( idet, 0.0d0 )
         do i = 1, nip
            zdet = zdet * zma( i, i )
         end do

c        Extract trace of zmd which is log derivative of zeta.
         ztrace = dcmplx( 0.0d0, 0.0d0 )
         do i = 1, nip
            ztrace = ztrace + zmd( i, i )
         end do

c        Store zeta and zlogd in appropriate array elements.
         zz( iz ) = zdet
         zd( iz ) = ztrace

         if( nip .le. 8 ) then
c           Print matrices.

            write( 6, * ) ' factorized zma = '
            do i = 1, nip
               write( 6, * ) ( zma( i, j ), j = 1, nip )
            end do

            write( 6, * ) ' solution zmd = '
            do i = 1, nip
               write( 6, * ) ( zmd( i, j ), j = 1, nip )
            end do
   
         end if


c        End of do iz = 1, nz loop over points of evaluation.
      end do

      if( ni .le. 16 ) then
c        Print block structure matrix.
         do ii = 1, ni
1000        format( 1x, 32(i2) )
            write( 6, 1000 ) ( istruct( ii, ji ), ji = 1, ni )
         end do
      end if

      return
      end
c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine dpsinit(iunit,lbox,lcap)
c     This procedure initializes PostScript graphics on unit number 
c     iunit, with a box around the drawing area if lbox is .true.

      implicit double precision(a-h,o-y),logical (l),complex (z)

c     Prologue to PS file; definitions.
      if(lcap)then
c        Set bounding box to include caption.
         write( unit = iunit, fmt = 1001)
1001     format(
     $    '%!PS'/
     $    '%%BoundingBox: 36 36 576 756')
      else
c        Set bounding box to not include caption.
         write( unit = iunit, fmt = 1002)
1002     format(
     $    '%!PS'/
     $    '%%BoundingBox: 36 216 576 756')
      end if

      write( unit = iunit, fmt = 1003)
1003  format(
     $ 'save'/
     $ '/l {lineto} bind def'/ '/n {newpath} bind def'/
     $ '/m {moveto} bind def'/
     $ '/t {stroke} bind def'/
     $ '/g {setgray} bind def'/
     $ '/f {fill} bind def'/
     $ '/c {closepath} bind def'/
     $ '/h {show} bind def'/
     $ '/r {rlineto} bind def'/
     $ '/rm {rmoveto} bind def'
     $ )

c     Origin and scaling transformations, and line parameters.
      write( unit = iunit, fmt = 1004)
1004  format(
     $ '36 216 translate'/
     $ '/SF 540 def SF SF scale'/
     $ '1 setlinecap'/
     $ '1 setlinejoin'
     $ )

c     Draw a box around drawing area.
      if(lbox) write( unit = iunit, fmt = 1005)
1005  format(
     $ '0.002 setlinewidth'/
     $ '0 0 m'/
     $ '0 1 l'/
     $ '1 1 l'/
     $ '1 0 l'/
     $ '0 0 l'/
     $ 't'
     $ )

c     Mark end of dpsinit.f output.
      write( unit = iunit, fmt = 1006)
1006  format('%End of dpsinit.f output')

      return
      end
      subroutine dpsterm(iunit)
c     This procedure terminates PS graphics on unit number iunit.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      write( unit = iunit, fmt = 1001)
1001  format(
     $ '%Beginning of dpsterm.f output'/
     $ 'restore'/
     $ 'showpage'/
     $ '%End of dpsterm.f output'
     $ )
      return
      end
      subroutine dpsnumfile(num,ipos,name,iunit)
c     This file keeps track of the files used for output;
c     it assumes that the name of the file is input in the
c     character variable 'name' and that the number 'num' is to be imbedded 
c     in 'name' in positions ipos through ipos+3, so the filename
c     has the form abcabcabc1234.defdfef where the number begins
c     in position ipos. The function of this routine is to 
c     put 'num' into 'name', increment it by one,
c     and open a file named 'name' on Fortran unit number iunit.
c     Typical usage is to use a series of individual PS pages
c     in files fig.1000.ps, fig.1001.ps,...  in which case ipos = 5.
c     The reason for starting at 1000
c     instead of 0000 is to avoid leading zero usage conflicts in shell scripts.
c     If iunit is negative, then output is not desired and the unix
c     file /dev/null is opened on unit number min(abs(iunit),99).

      implicit double precision (a-h,o-y), complex(z)
      character*(*) name
      if(iunit.gt.0.and.iunit.lt.100)then
c        Unpack num into four characters.
         i3 = num/1000
         i2 = num/100 -10  *i3
         i1 = num/10  -100 *i3-10 *i2
         i0 = num/1   -1000*i3-100*i2-10*i1
         name(ipos:ipos+3) = 
     $  char(48+i3)//char(48+i2)//char(48+i1)//char(48+i0)
         num = num+1
c        Open table file on unit iunit.
         open(unit = iunit,form = 'formatted',status = 'unknown',
     $     file = name,access = 'sequential')
      else
	 iunit = min(abs(iunit),99)
c        Open unix trash bin on unit |iunit|.
         open(unit = iunit,form = 'formatted',status = 'unknown',
     $     file = '/dev/null',access = 'sequential')
      end if
      return
      end
      subroutine dpscurve(iunit,n,x,y,xl,xr,yl,yr,thickness,gray)
c     This routine draws a curve of n points given in x and y on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) thickness,gray
1001  format(' gsave ',f6.4,' setlinewidth ',f6.4,' g')

      i = 1
      xx = (x(i)-xl)*xlen
      yy = (y(i)-yl)*ylen
      write( unit = iunit, fmt = 1002) xx,yy
1002  format(f6.4,1x,f6.4,' m')

      do 10 i = 2,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         write( unit = iunit, fmt = 1003) xx,yy
10    continue
1003  format(f6.4,1x,f6.4,' l')

c     Stroke curve.
      write( unit = iunit, fmt = 1004)
1004  format('t')

      write( unit = iunit, fmt = 1005)
1005  format(' grestore')

      return
      end

      subroutine dpsdot(iunit,n,x,y,xl,xr,yl,yr,radius,gray)
c     This routine draws dots of radius 'radius' at n points given
c     in array x and y, in grayscale color 'gray (1 = white,0 = black).
c     The drawing area is [0,1]^2 and x and y are scaled by xr,xl,yr,yl.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) gray,radius
1001  format('gsave ',f6.4,' g'/
     $ '/d { ',f6.4,' 0 360 arc f} bind def'/
     $ 'n'
     $ )

      do 10 i = 1,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         write( unit = iunit, fmt = 1002) xx,yy
10    continue
1002  format(f6.4,1x,f6.4,' d')
      write( unit = iunit, fmt = 1003)
1003  format(' t grestore')

      return
      end

      subroutine dpsvdot(iunit,n,x,y,xl,xr,yl,yr,radius,gray)
c     This routine draws dots of variable radii 'radius' at n points given
c     in array x and y, in grayscale color 'gray (1 = white,0 = black).
c     The drawing area is [0,1]^2 and x and y are scaled by xr,xl,yr,yl.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),radius(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) gray
1001  format('gsave ',f6.4,' g'/
     $ '/d {0 360 arc f} bind def'/
     $ 'n'
     $ )

c     Draw individual dots.
      radscal = max(xlen,ylen)
      do 10 i = 1,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         rr = radius(i)*radscal
         write( unit = iunit, fmt = 1002) xx,yy,radius(i)
10    continue
1002  format(f6.4,1x,f6.4,1x,f6.4,' d')
c     Finish drawing.
      write( unit = iunit, fmt = 1003)
1003  format(' t grestore')

      return
      end

      subroutine dpsseg(iunit,x1,y1,x2,y2,xl,xr,yl,yr)
c     This routine draws a segment of 2 points given in x and y on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white.  These parameters are set in dpsseginit.
c     Segments lying completely outside the drawing area are skipped.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      xlen = 1.0/(xr-xl)
      if(xlen.le.0.0)return
      ylen = 1.0/(yr-yl)
      if(ylen.le.0.0)return
c     Skip segments outside drawing area.
      if(max(x1,x2).lt.xl)return
      if(min(x1,x2).gt.xr)return
      if(max(y1,y2).lt.yl)return
      if(min(y1,y2).gt.yr)return

      xx = (x1-xl)*xlen
      yy = (y1-yl)*ylen
      write( unit = iunit, fmt = 1002) xx,yy
1002  format(f6.4,1x,f6.4,' m')

      xx = (x2-xl)*xlen
      yy = (y2-yl)*ylen
      write( unit = iunit, fmt = 1003) xx,yy
1003  format(f6.4,1x,f6.4,' l')

c     Stroke curve.
      write( unit = iunit, fmt = 1004)
1004  format('t')

      return
      end
      subroutine dpsgraphinit(iunit,xl,xr,xa,yl,yr,ya,
     $ th,grayl,nxt,nyt,xt,yt,cx,cy,ipsize)
c     This routine initializes a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      character cx*(*),cy*(*)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) 
     >   (xa-xl)*xlen,(ya-yl)*ylen,th,ipsize
1001  format('%Begin dpsgraphinit.f output'/
     $ 'gsave /XM 0.10 def /YM 0.10 def'/
     $ ' /XS 0.85 def /YS 0.85 def '/
     $ ' /XA ',f6.4,' def /YA ',f6.4,' def '/
     $ 'XM YM translate XS YS scale'/
     $ f6.4,' setlinewidth'/
     $ '/Helvetica findfont /PS ',i4,
     > ' SF div def PS scalefont setfont'/
     $ '/LS 1.25 PS mul def ')
c     Draw box around graph.
      write( unit = iunit, fmt = 1020)
1020  format('0 0 m 0 1 r 1 0 r 0 -1 r closepath t')
      if(xl.le.0.0.and.xr.ge.0.0)then
c     Draw x-axis crossing at x = xa.
      write( unit = iunit, fmt = 1002) 
1002  format(' XA 0 m 0 1 r t')
      end if
      if(yl.le.0.0.and.yr.ge.0.0)then
c     Draw y-axis crossing at y = 0.
      write( unit = iunit, fmt = 1003)
1003  format(' 0 YA m 1 0 r t')
      end if

c     Tick marks on x-axis.
      if(nxt.gt.0)then
      write( unit = iunit, fmt = 1004) xt
1004  format('/txl {dup YA m 0 ',f12.4,
     $ ' r t YA LS sub m dup stringwidth pop ',
     $ '2 div neg 0 rm h } def')
      do 100 ixt = 0,nxt
ccc      do 100 ixt = 1,nxt
      xx = dble(ixt)/dble(nxt)
c           write( unit = iunit, fmt = 1005) xx*(xr-xl)+xl,xx
      write( unit = iunit, fmt = 1005) nint(xx*(xr-xl)+xl),xx
100   continue
c     1005  format('(',f9.2,') ',g10.4,' txl')
1005  format('(',i2,') ',g10.4,' txl')
      end if

c     Caption for x-axis.
      write( unit = iunit, fmt = 1007) cx
1007  format('0.5 YM 0.66 mul neg  m (',a,')'/
     $ ' dup stringwidth pop 2 div neg PS 2 div neg rm h')

c     Tick marks on y-axis.
      if(nyt.gt.0)then
      write( unit = iunit, fmt = 1008) yt
1008  format('/tyl { dup XA exch m ',f6.4,' 0 r t'/
     $ 'XA exch m dup stringwidth pop neg PS 2 div neg rm h } def')
ccc      do 200 iyt = 0,nyt
      do 200 iyt = 1,nyt
      yy = dble(iyt)/dble(nyt)
c      write( unit = iunit, fmt = 1009) yy*(yr-yl)+yl,yy
      write( unit = iunit, fmt = 1009) nint(yy*(yr-yl)+yl),yy
200   continue
c1009  format('(',f6.1,' ) ',f6.4,' tyl')
1009  format('(',i2,' ) ',f6.4,' tyl')
      end if

c     Caption for y-axis.
      write( unit = iunit, fmt = 1011) cy
1011  format(' gsave 90 rotate 0.5 XM 0.66 mul m (',a,')'/
     $ ' dup stringwidth pop 2 div neg PS 1.5 mul rm h grestore'/
     $ ' %End dpsgraphinit.f output')

      return
      end
      subroutine dpsgraphterm(iunit)
c     This routine terminates a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)

      write( unit = iunit, fmt = 1013)
1013  format('%Begin dpsgraphterm.f output'/
     $  'grestore'/
     $  '%End dpsgraphterm.f output')
      
      return
      end

      subroutine dpsgraph(iunit,n,x,y,xl,xr,yl,yr,
     $ radius,dotgray,ndash,dash,th,grayl)
c     This routine draws a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.
c     The line is drawn with dash pattern specified by ndash and dash
c     according to the postscript usage; ndash <11 is the array length,
c     the elements of dash specify the dash pattern.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),dash(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(radius.gt.0.0)then
      call dpsdot(iunit,n,x,y,xl,xr,yl,yr,radius,dotgray)
      end if

      if(th.gt.0.0)then
      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if
      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do 300 i = 1,n-1
      call dpsseg(iunit,x(i),y(i),x(i+1),y(i+1),xl,xr,yl,yr)
300   continue
      call dpssegterm(iunit)
      end if

      return
      end

      subroutine dpsgraphs( iunit, n, m, x, nq, y, xl, xr, yl, yr, 
     >   ndash, dash, nxt, th, grayl )
c     This routine draws a graph of nq variables y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given and y data at each in y(1:nq, * ) .
c     Tick marks are drawn at min( n, nxt ) data points on the x-axis.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.
c     The line is drawn with dash pattern specified by ndash and dash
c     according to the postscript usage; ndash <11 is the array length,
c     the elements of dash specify the dash pattern.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(nq,1),dash(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)

c     Number of tick marks on x-axis.
      nt = min( n, nxt )
c     Frequency of tick marks on x-axis.
      mt = max( 1, n / max( 1, nt ) )

c     Number of points to plot.
      np = min( n, m )
c     Frequency of points to plot.
      mp = max( 1, n / np )

      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(th.gt.0.0)then

      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if

      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do iq = 1, nq
         do i = 1, n - 1, mp
            xb = x( i )
            yb = y( iq, i ) 
            ie = min( n, i + mp )
            xe = x( ie )
            ye = y( iq, ie )
            call dpsseg( iunit, xb, yb, xe, ye, xl, xr, yl, yr )
         end do
      end do

c     Draw ticks vertically at location x(i), covering 2% of the vertical span.
      do i = 1, n, mt
         call dpsseg( iunit, x( i ), yl, x( i ), yl + 
     >      0.02 * ( yr - yl ), xl, xr, yl, yr )
      end do

      call dpssegterm(iunit)
      end if

      return
      end

      subroutine dpsgraphletter( iunit, n, x, y, xl, xr, yl, yr,
     >      c, ndash, dash, th, grayl )
c     This routine letterplots a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),dash(1)
      character*1 c(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(th.gt.0.0)then
      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if
      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do 300 i = 1,n-1
      call dpsseg( iunit, x(i), y(i), x(i+1), y(i+1), xl, xr, yl, yr)
300   continue
      call dpssegterm(iunit)
      end if

      do i = 1, n
 
         xx = ( x( i ) - xl ) * xlen
         yy = ( y( i ) - yl ) * ylen

         write( unit = iunit, fmt = 1012) xx, yy, c( i )
1012     format(f6.4, 1x ,f6.4, ' m (', a1, ') h' )

      end do

      return
      end

      subroutine dpsseginit(iunit,xl,xr,yl,yr,thickness,gray)
c     This routine initializes segment drawing routine dpsseg on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) thickness,gray
1001  format(' gsave ',f6.4,' setlinewidth ',f6.4,' g')

      return
      end
      subroutine dpssegterm(iunit)
c     This routine terminates segment drawing routine dpsseg on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      write( unit = iunit, fmt = 1005)
1005  format(' grestore')
      return
      end
      subroutine dpscapinit(iunit,ipsize,lbox)
c     This routine initializes caption drawing in the lower quarter
c     of the page. If lbox is true, a box is drawn around it.
c     Routine dpsinit should be called before this routine,
c     and dpsterm should be called to complete the file;
c     no routine is necessary for terminating the caption.
c     Parameter ipsize is the point size for drawing the caption.
c     Parameter iunit is the unit number on which the figure is written.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      if(lbox) write( unit = iunit, fmt = 1001)
1001  format(
     $ '%Begin dpscapinit.f output'/
     $ '%Box around caption area'/
     $ '0 0 m'/
     $ '1 0 l'/
     $ '1 -.333333 l'/
     $ '0 -.333333 l'/
     $ '0 0 l'/
     $ 't'
     $ )

c     The following lines choose a font (Helvetica), scale it
c     to point size PS = ipsize, and set the line spacing LS,
c     left margin LM and caption line position CL.
c     The point size is divided by the scale factor SF set by dpsinit,
c     and the other parameters are defined in terms of PS.
c     We then define a PS procedure capline which writes a string
c     and then moves the current point down one line spacing.
      write( unit = iunit, fmt = 1002) ipsize
1002  format(
     $ '%Drawing captions.'/
     $ '%Font used must be scaled appropriately.'/
     $ '/Helvetica findfont '/
     $ '/PS ',i3,' SF div def'/
     $ 'PS scalefont setfont'/
     $ '/LS 1.25 PS mul def'/
     $ '/LM  LS def'/
     $ '/CL  LS neg def'/
     $ '/capline {LM CL m h /CL CL LS sub def} def'/
     $ '%End dpscapinit.f output'
     $ )

      return
      end
      subroutine dpscapline(iunit,line)
c     This routine writes one line of the caption, in the lower
c     quarter of the drawing area. It takes a character variable
c     'line' of length less than 50 characters; later I should 
c     do something a little more general.
c     iunit is the unit number on which the PS file is written.
c     Procedure dpscapinit should be called first.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      character*(*) line
      write( unit = iunit, fmt = 1001) line
1001  format(1x,'(',a,') capline')
      return
      end
      double precision function dcabs1(z)
      double complex z,zz
      double precision t(2)
      equivalence (zz,t(1))
      zz = z
      dcabs1 = dabs(t(1)) + dabs(t(2))
      return
      end
      INTEGER          FUNCTION IEEECK( ISPEC, ZERO, ONE )
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1998
*
*     .. Scalar Arguments ..
      INTEGER            ISPEC
      REAL               ONE, ZERO
*     ..
*
*  Purpose
*  =======
*
*  IEEECK is called from the ILAENV to verify that Infinity and
*  possibly NaN arithmetic is safe (i.e. will not trap).
*
*  Arguments
*  =========
*
*  ISPEC   (input) INTEGER
*          Specifies whether to test just for inifinity arithmetic
*          or whether to test for infinity and NaN arithmetic.
*          = 0: Verify infinity arithmetic only.
*          = 1: Verify infinity and NaN arithmetic.
*
*  ZERO    (input) REAL
*          Must contain the value 0.0
*          This is passed to prevent the compiler from optimizing
*          away this code.
*
*  ONE     (input) REAL
*          Must contain the value 1.0
*          This is passed to prevent the compiler from optimizing
*          away this code.
*
*  RETURN VALUE:  INTEGER
*          = 0:  Arithmetic failed to produce the correct answers
*          = 1:  Arithmetic produced the correct answers
*
*     .. Local Scalars ..
      REAL               NAN1, NAN2, NAN3, NAN4, NAN5, NAN6, NEGINF,
     $                   NEGZRO, NEWZRO, POSINF
*     ..
*     .. Executable Statements ..
      IEEECK = 1
*
      POSINF = ONE / ZERO
      IF( POSINF.LE.ONE ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      NEGINF = -ONE / ZERO
      IF( NEGINF.GE.ZERO ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      NEGZRO = ONE / ( NEGINF+ONE )
      IF( NEGZRO.NE.ZERO ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      NEGINF = ONE / NEGZRO
      IF( NEGINF.GE.ZERO ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      NEWZRO = NEGZRO + ZERO
      IF( NEWZRO.NE.ZERO ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      POSINF = ONE / NEWZRO
      IF( POSINF.LE.ONE ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      NEGINF = NEGINF*POSINF
      IF( NEGINF.GE.ZERO ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      POSINF = POSINF*POSINF
      IF( POSINF.LE.ONE ) THEN
         IEEECK = 0
         RETURN
      END IF
*
*
*
*
*     Return if we were only asked to check infinity arithmetic
*
      IF( ISPEC.EQ.0 )
     $   RETURN
*
      NAN1 = POSINF + NEGINF
*
      NAN2 = POSINF / NEGINF
*
      NAN3 = POSINF / POSINF
*
      NAN4 = POSINF*ZERO
*
      NAN5 = NEGINF*NEGZRO
*
      NAN6 = NAN5*0.0
*
      IF( NAN1.EQ.NAN1 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      IF( NAN2.EQ.NAN2 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      IF( NAN3.EQ.NAN3 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      IF( NAN4.EQ.NAN4 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      IF( NAN5.EQ.NAN5 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      IF( NAN6.EQ.NAN6 ) THEN
         IEEECK = 0
         RETURN
      END IF
*
      RETURN
      END
      INTEGER          FUNCTION ILAENV( ISPEC, NAME, OPTS, N1, N2, N3,
     $                 N4 )
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1999
*
*     .. Scalar Arguments ..
      CHARACTER*( * )    NAME, OPTS
      INTEGER            ISPEC, N1, N2, N3, N4
*     ..
*
*  Purpose
*  =======
*
*  ILAENV is called from the LAPACK routines to choose problem-dependent
*  parameters for the local environment.  See ISPEC for a description of
*  the parameters.
*
*  This version provides a set of parameters which should give good,
*  but not optimal, performance on many of the currently available
*  computers.  Users are encouraged to modify this subroutine to set
*  the tuning parameters for their particular machine using the option
*  and problem size information in the arguments.
*
*  This routine will not function correctly if it is converted to all
*  lower case.  Converting it to all upper case is allowed.
*
*  Arguments
*  =========
*
*  ISPEC   (input) INTEGER
*          Specifies the parameter to be returned as the value of
*          ILAENV.
*          = 1: the optimal blocksize; if this value is 1, an unblocked
*               algorithm will give the best performance.
*          = 2: the minimum block size for which the block routine
*               should be used; if the usable block size is less than
*               this value, an unblocked routine should be used.
*          = 3: the crossover point (in a block routine, for N less
*               than this value, an unblocked routine should be used)
*          = 4: the number of shifts, used in the nonsymmetric
*               eigenvalue routines
*          = 5: the minimum column dimension for blocking to be used;
*               rectangular blocks must have dimension at least k by m,
*               where k is given by ILAENV(2,...) and m by ILAENV(5,...)
*          = 6: the crossover point for the SVD (when reducing an m by n
*               matrix to bidiagonal form, if max(m,n)/min(m,n) exceeds
*               this value, a QR factorization is used first to reduce
*               the matrix to a triangular form.)
*          = 7: the number of processors
*          = 8: the crossover point for the multishift QR and QZ methods
*               for nonsymmetric eigenvalue problems.
*          = 9: maximum size of the subproblems at the bottom of the
*               computation tree in the divide-and-conquer algorithm
*               (used by xGELSD and xGESDD)
*          =10: ieee NaN arithmetic can be trusted not to trap
*          =11: infinity arithmetic can be trusted not to trap
*
*  NAME    (input) CHARACTER*(*)
*          The name of the calling subroutine, in either upper case or
*          lower case.
*
*  OPTS    (input) CHARACTER*(*)
*          The character options to the subroutine NAME, concatenated
*          into a single character string.  For example, UPLO = 'U',
*          TRANS = 'T', and DIAG = 'N' for a triangular routine would
*          be specified as OPTS = 'UTN'.
*
*  N1      (input) INTEGER
*  N2      (input) INTEGER
*  N3      (input) INTEGER
*  N4      (input) INTEGER
*          Problem dimensions for the subroutine NAME; these may not all
*          be required.
*
* (ILAENV) (output) INTEGER
*          >= 0: the value of the parameter specified by ISPEC
*          < 0:  if ILAENV = -k, the k-th argument had an illegal value.
*
*  Further Details
*  ===============
*
*  The following conventions have been used when calling ILAENV from the
*  LAPACK routines:
*  1)  OPTS is a concatenation of all of the character options to
*      subroutine NAME, in the same order that they appear in the
*      argument list for NAME, even if they are not used in determining
*      the value of the parameter specified by ISPEC.
*  2)  The problem dimensions N1, N2, N3, N4 are specified in the order
*      that they appear in the argument list for NAME.  N1 is used
*      first, N2 second, and so on, and unused problem dimensions are
*      passed a value of -1.
*  3)  The parameter value returned by ILAENV is checked for validity in
*      the calling subroutine.  For example, ILAENV is used to retrieve
*      the optimal blocksize for STRTRI as follows:
*
*      NB = ILAENV( 1, 'STRTRI', UPLO // DIAG, N, -1, -1, -1 )
*      IF( NB.LE.1 ) NB = MAX( 1, N )
*
*  =====================================================================
*
*     .. Local Scalars ..
      LOGICAL            CNAME, SNAME
      CHARACTER*1        C1
      CHARACTER*2        C2, C4
      CHARACTER*3        C3
      CHARACTER*6        SUBNAM
      INTEGER            I, IC, IZ, NB, NBMIN, NX
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          CHAR, ICHAR, INT, MIN, REAL
*     ..
*     .. External Functions ..
      INTEGER            IEEECK
      EXTERNAL           IEEECK
*     ..
*     .. Executable Statements ..
*
      GO TO ( 100, 100, 100, 400, 500, 600, 700, 800, 900, 1000,
     $        1100 ) ISPEC
*
*     Invalid value for ISPEC
*
      ILAENV = -1
      RETURN
*
  100 CONTINUE
*
*     Convert NAME to upper case if the first character is lower case.
*
      ILAENV = 1
      SUBNAM = NAME
      IC = ICHAR( SUBNAM( 1:1 ) )
      IZ = ICHAR( 'Z' )
      IF( IZ.EQ.90 .OR. IZ.EQ.122 ) THEN
*
*        ASCII character set
*
         IF( IC.GE.97 .AND. IC.LE.122 ) THEN
            SUBNAM( 1:1 ) = CHAR( IC-32 )
            DO 10 I = 2, 6
               IC = ICHAR( SUBNAM( I:I ) )
               IF( IC.GE.97 .AND. IC.LE.122 )
     $            SUBNAM( I:I ) = CHAR( IC-32 )
   10       CONTINUE
         END IF
*
      ELSE IF( IZ.EQ.233 .OR. IZ.EQ.169 ) THEN
*
*        EBCDIC character set
*
         IF( ( IC.GE.129 .AND. IC.LE.137 ) .OR.
     $       ( IC.GE.145 .AND. IC.LE.153 ) .OR.
     $       ( IC.GE.162 .AND. IC.LE.169 ) ) THEN
            SUBNAM( 1:1 ) = CHAR( IC+64 )
            DO 20 I = 2, 6
               IC = ICHAR( SUBNAM( I:I ) )
               IF( ( IC.GE.129 .AND. IC.LE.137 ) .OR.
     $             ( IC.GE.145 .AND. IC.LE.153 ) .OR.
     $             ( IC.GE.162 .AND. IC.LE.169 ) )
     $            SUBNAM( I:I ) = CHAR( IC+64 )
   20       CONTINUE
         END IF
*
      ELSE IF( IZ.EQ.218 .OR. IZ.EQ.250 ) THEN
*
*        Prime machines:  ASCII+128
*
         IF( IC.GE.225 .AND. IC.LE.250 ) THEN
            SUBNAM( 1:1 ) = CHAR( IC-32 )
            DO 30 I = 2, 6
               IC = ICHAR( SUBNAM( I:I ) )
               IF( IC.GE.225 .AND. IC.LE.250 )
     $            SUBNAM( I:I ) = CHAR( IC-32 )
   30       CONTINUE
         END IF
      END IF
*
      C1 = SUBNAM( 1:1 )
      SNAME = C1.EQ.'S' .OR. C1.EQ.'D'
      CNAME = C1.EQ.'C' .OR. C1.EQ.'Z'
      IF( .NOT.( CNAME .OR. SNAME ) )
     $   RETURN
      C2 = SUBNAM( 2:3 )
      C3 = SUBNAM( 4:6 )
      C4 = C3( 2:3 )
*
      GO TO ( 110, 200, 300 ) ISPEC
*
  110 CONTINUE
*
*     ISPEC = 1:  block size
*
*     In these examples, separate code is provided for setting NB for
*     real and complex.  We assume that NB will take the same value in
*     single or double precision.
*
      NB = 1
*
      IF( C2.EQ.'GE' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         ELSE IF( C3.EQ.'QRF' .OR. C3.EQ.'RQF' .OR. C3.EQ.'LQF' .OR.
     $            C3.EQ.'QLF' ) THEN
            IF( SNAME ) THEN
               NB = 32
            ELSE
               NB = 32
            END IF
         ELSE IF( C3.EQ.'HRD' ) THEN
            IF( SNAME ) THEN
               NB = 32
            ELSE
               NB = 32
            END IF
         ELSE IF( C3.EQ.'BRD' ) THEN
            IF( SNAME ) THEN
               NB = 32
            ELSE
               NB = 32
            END IF
         ELSE IF( C3.EQ.'TRI' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         END IF
      ELSE IF( C2.EQ.'PO' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         END IF
      ELSE IF( C2.EQ.'SY' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         ELSE IF( SNAME .AND. C3.EQ.'TRD' ) THEN
            NB = 32
         ELSE IF( SNAME .AND. C3.EQ.'GST' ) THEN
            NB = 64
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'HE' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            NB = 64
         ELSE IF( C3.EQ.'TRD' ) THEN
            NB = 32
         ELSE IF( C3.EQ.'GST' ) THEN
            NB = 64
         END IF
      ELSE IF( SNAME .AND. C2.EQ.'OR' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NB = 32
            END IF
         ELSE IF( C3( 1:1 ).EQ.'M' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NB = 32
            END IF
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'UN' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NB = 32
            END IF
         ELSE IF( C3( 1:1 ).EQ.'M' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NB = 32
            END IF
         END IF
      ELSE IF( C2.EQ.'GB' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               IF( N4.LE.64 ) THEN
                  NB = 1
               ELSE
                  NB = 32
               END IF
            ELSE
               IF( N4.LE.64 ) THEN
                  NB = 1
               ELSE
                  NB = 32
               END IF
            END IF
         END IF
      ELSE IF( C2.EQ.'PB' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               IF( N2.LE.64 ) THEN
                  NB = 1
               ELSE
                  NB = 32
               END IF
            ELSE
               IF( N2.LE.64 ) THEN
                  NB = 1
               ELSE
                  NB = 32
               END IF
            END IF
         END IF
      ELSE IF( C2.EQ.'TR' ) THEN
         IF( C3.EQ.'TRI' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         END IF
      ELSE IF( C2.EQ.'LA' ) THEN
         IF( C3.EQ.'UUM' ) THEN
            IF( SNAME ) THEN
               NB = 64
            ELSE
               NB = 64
            END IF
         END IF
      ELSE IF( SNAME .AND. C2.EQ.'ST' ) THEN
         IF( C3.EQ.'EBZ' ) THEN
            NB = 1
         END IF
      END IF
      ILAENV = NB
      RETURN
*
  200 CONTINUE
*
*     ISPEC = 2:  minimum block size
*
      NBMIN = 2
      IF( C2.EQ.'GE' ) THEN
         IF( C3.EQ.'QRF' .OR. C3.EQ.'RQF' .OR. C3.EQ.'LQF' .OR.
     $       C3.EQ.'QLF' ) THEN
            IF( SNAME ) THEN
               NBMIN = 2
            ELSE
               NBMIN = 2
            END IF
         ELSE IF( C3.EQ.'HRD' ) THEN
            IF( SNAME ) THEN
               NBMIN = 2
            ELSE
               NBMIN = 2
            END IF
         ELSE IF( C3.EQ.'BRD' ) THEN
            IF( SNAME ) THEN
               NBMIN = 2
            ELSE
               NBMIN = 2
            END IF
         ELSE IF( C3.EQ.'TRI' ) THEN
            IF( SNAME ) THEN
               NBMIN = 2
            ELSE
               NBMIN = 2
            END IF
         END IF
      ELSE IF( C2.EQ.'SY' ) THEN
         IF( C3.EQ.'TRF' ) THEN
            IF( SNAME ) THEN
               NBMIN = 8
            ELSE
               NBMIN = 8
            END IF
         ELSE IF( SNAME .AND. C3.EQ.'TRD' ) THEN
            NBMIN = 2
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'HE' ) THEN
         IF( C3.EQ.'TRD' ) THEN
            NBMIN = 2
         END IF
      ELSE IF( SNAME .AND. C2.EQ.'OR' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NBMIN = 2
            END IF
         ELSE IF( C3( 1:1 ).EQ.'M' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NBMIN = 2
            END IF
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'UN' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NBMIN = 2
            END IF
         ELSE IF( C3( 1:1 ).EQ.'M' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NBMIN = 2
            END IF
         END IF
      END IF
      ILAENV = NBMIN
      RETURN
*
  300 CONTINUE
*
*     ISPEC = 3:  crossover point
*
      NX = 0
      IF( C2.EQ.'GE' ) THEN
         IF( C3.EQ.'QRF' .OR. C3.EQ.'RQF' .OR. C3.EQ.'LQF' .OR.
     $       C3.EQ.'QLF' ) THEN
            IF( SNAME ) THEN
               NX = 128
            ELSE
               NX = 128
            END IF
         ELSE IF( C3.EQ.'HRD' ) THEN
            IF( SNAME ) THEN
               NX = 128
            ELSE
               NX = 128
            END IF
         ELSE IF( C3.EQ.'BRD' ) THEN
            IF( SNAME ) THEN
               NX = 128
            ELSE
               NX = 128
            END IF
         END IF
      ELSE IF( C2.EQ.'SY' ) THEN
         IF( SNAME .AND. C3.EQ.'TRD' ) THEN
            NX = 32
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'HE' ) THEN
         IF( C3.EQ.'TRD' ) THEN
            NX = 32
         END IF
      ELSE IF( SNAME .AND. C2.EQ.'OR' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NX = 128
            END IF
         END IF
      ELSE IF( CNAME .AND. C2.EQ.'UN' ) THEN
         IF( C3( 1:1 ).EQ.'G' ) THEN
            IF( C4.EQ.'QR' .OR. C4.EQ.'RQ' .OR. C4.EQ.'LQ' .OR.
     $          C4.EQ.'QL' .OR. C4.EQ.'HR' .OR. C4.EQ.'TR' .OR.
     $          C4.EQ.'BR' ) THEN
               NX = 128
            END IF
         END IF
      END IF
      ILAENV = NX
      RETURN
*
  400 CONTINUE
*
*     ISPEC = 4:  number of shifts (used by xHSEQR)
*
      ILAENV = 6
      RETURN
*
  500 CONTINUE
*
*     ISPEC = 5:  minimum column dimension (not used)
*
      ILAENV = 2
      RETURN
*
  600 CONTINUE 
*
*     ISPEC = 6:  crossover point for SVD (used by xGELSS and xGESVD)
*
      ILAENV = INT( REAL( MIN( N1, N2 ) )*1.6E0 )
      RETURN
*
  700 CONTINUE
*
*     ISPEC = 7:  number of processors (not used)
*
      ILAENV = 1
      RETURN
*
  800 CONTINUE
*
*     ISPEC = 8:  crossover point for multishift (used by xHSEQR)
*
      ILAENV = 50
      RETURN
*
  900 CONTINUE
*
*     ISPEC = 9:  maximum size of the subproblems at the bottom of the
*                 computation tree in the divide-and-conquer algorithm
*                 (used by xGELSD and xGESDD)
*
      ILAENV = 25
      RETURN
*
 1000 CONTINUE
*
*     ISPEC = 10: ieee NaN arithmetic can be trusted not to trap
*
C     ILAENV = 0
      ILAENV = 1
      IF( ILAENV.EQ.1 ) THEN
         ILAENV = IEEECK( 0, 0.0, 1.0 ) 
      END IF
      RETURN
*
 1100 CONTINUE
*
*     ISPEC = 11: infinity arithmetic can be trusted not to trap
*
C     ILAENV = 0
      ILAENV = 1
      IF( ILAENV.EQ.1 ) THEN
         ILAENV = IEEECK( 1, 0.0, 1.0 ) 
      END IF
      RETURN
*
*     End of ILAENV
*
      END
      integer function izamax(n,zx,incx)
c
c     finds the index of element having max. absolute value.
c     jack dongarra, 1/15/85.
c     modified 3/93 to return if incx .le. 0.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double complex zx(*)
      double precision smax
      integer i,incx,ix,n
      double precision dcabs1
c
      izamax = 0
      if( n.lt.1 .or. incx.le.0 )return
      izamax = 1
      if(n.eq.1)return
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c
      ix = 1
      smax = dcabs1(zx(1))
      ix = ix + incx
      do 10 i = 2,n
         if(dcabs1(zx(ix)).le.smax) go to 5
         izamax = i
         smax = dcabs1(zx(ix))
    5    ix = ix + incx
   10 continue
      return
c
c        code for increment equal to 1
c
   20 smax = dcabs1(zx(1))
      do 30 i = 2,n
         if(dcabs1(zx(i)).le.smax) go to 30
         izamax = i
         smax = dcabs1(zx(i))
   30 continue
      return
      end
      SUBROUTINE XERBLA( SRNAME, INFO )
*
*  -- LAPACK auxiliary routine (preliminary version) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     February 29, 1992
*
*     .. Scalar Arguments ..
      CHARACTER*6        SRNAME
      INTEGER            INFO
*     ..
*
*  Purpose
*  =======
*
*  XERBLA  is an error handler for the LAPACK routines.
*  It is called by an LAPACK routine if an input parameter has an
*  invalid value.  A message is printed and execution stops.
*
*  Installers may consider modifying the STOP statement in order to
*  call system-specific exception-handling facilities.
*
*  Arguments
*  =========
*
*  SRNAME  (input) CHARACTER*6
*          The name of the routine which called XERBLA.
*
*  INFO    (input) INTEGER
*          The position of the invalid parameter in the parameter list
*          of the calling routine.
*
*
      WRITE( *, FMT = 9999 )SRNAME, INFO
*
      STOP
*
 9999 FORMAT( ' ** On entry to ', A6, ' parameter number ', I2, ' had ',
     $      'an illegal value' )
*
*     End of XERBLA
*
      END
      SUBROUTINE ZGEMM ( TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, B, LDB,
     $                   BETA, C, LDC )
*     .. Scalar Arguments ..
      CHARACTER*1        TRANSA, TRANSB
      INTEGER            M, N, K, LDA, LDB, LDC
      COMPLEX*16         ALPHA, BETA
*     .. Array Arguments ..
      COMPLEX*16         A( LDA, * ), B( LDB, * ), C( LDC, * )
*     ..
*
*  Purpose
*  =======
*
*  ZGEMM  performs one of the matrix-matrix operations
*
*     C := alpha*op( A )*op( B ) + beta*C,
*
*  where  op( X ) is one of
*
*     op( X ) = X   or   op( X ) = X'   or   op( X ) = conjg( X' ),
*
*  alpha and beta are scalars, and A, B and C are matrices, with op( A )
*  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
*
*  Parameters
*  ==========
*
*  TRANSA - CHARACTER*1.
*           On entry, TRANSA specifies the form of op( A ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSA = 'N' or 'n',  op( A ) = A.
*
*              TRANSA = 'T' or 't',  op( A ) = A'.
*
*              TRANSA = 'C' or 'c',  op( A ) = conjg( A' ).
*
*           Unchanged on exit.
*
*  TRANSB - CHARACTER*1.
*           On entry, TRANSB specifies the form of op( B ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSB = 'N' or 'n',  op( B ) = B.
*
*              TRANSB = 'T' or 't',  op( B ) = B'.
*
*              TRANSB = 'C' or 'c',  op( B ) = conjg( B' ).
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry,  M  specifies  the number  of rows  of the  matrix
*           op( A )  and of the  matrix  C.  M  must  be at least  zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry,  N  specifies the number  of columns of the matrix
*           op( B ) and the number of columns of the matrix C. N must be
*           at least zero.
*           Unchanged on exit.
*
*  K      - INTEGER.
*           On entry,  K  specifies  the number of columns of the matrix
*           op( A ) and the number of rows of the matrix op( B ). K must
*           be at least  zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, ka ), where ka is
*           k  when  TRANSA = 'N' or 'n',  and is  m  otherwise.
*           Before entry with  TRANSA = 'N' or 'n',  the leading  m by k
*           part of the array  A  must contain the matrix  A,  otherwise
*           the leading  k by m  part of the array  A  must contain  the
*           matrix A.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. When  TRANSA = 'N' or 'n' then
*           LDA must be at least  max( 1, m ), otherwise  LDA must be at
*           least  max( 1, k ).
*           Unchanged on exit.
*
*  B      - COMPLEX*16       array of DIMENSION ( LDB, kb ), where kb is
*           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
*           Before entry with  TRANSB = 'N' or 'n',  the leading  k by n
*           part of the array  B  must contain the matrix  B,  otherwise
*           the leading  n by k  part of the array  B  must contain  the
*           matrix B.
*           Unchanged on exit.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in the calling (sub) program. When  TRANSB = 'N' or 'n' then
*           LDB must be at least  max( 1, k ), otherwise  LDB must be at
*           least  max( 1, n ).
*           Unchanged on exit.
*
*  BETA   - COMPLEX*16      .
*           On entry,  BETA  specifies the scalar  beta.  When  BETA  is
*           supplied as zero then C need not be set on input.
*           Unchanged on exit.
*
*  C      - COMPLEX*16       array of DIMENSION ( LDC, n ).
*           Before entry, the leading  m by n  part of the array  C must
*           contain the matrix  C,  except when  beta  is zero, in which
*           case C need not be set on entry.
*           On exit, the array  C  is overwritten by the  m by n  matrix
*           ( alpha*op( A )*op( B ) + beta*C ).
*
*  LDC    - INTEGER.
*           On entry, LDC specifies the first dimension of C as declared
*           in  the  calling  (sub)  program.   LDC  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     .. External Subroutines ..
      EXTERNAL           XERBLA
*     .. Intrinsic Functions ..
      INTRINSIC          DCONJG, MAX
*     .. Local Scalars ..
      LOGICAL            CONJA, CONJB, NOTA, NOTB
      INTEGER            I, INFO, J, L, NCOLA, NROWA, NROWB
      COMPLEX*16         TEMP
*     .. Parameters ..
      COMPLEX*16         ONE
      PARAMETER        ( ONE  = ( 1.0D+0, 0.0D+0 ) )
      COMPLEX*16         ZERO
      PARAMETER        ( ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Executable Statements ..
*
*     Set  NOTA  and  NOTB  as  true if  A  and  B  respectively are not
*     conjugated or transposed, set  CONJA and CONJB  as true if  A  and
*     B  respectively are to be  transposed but  not conjugated  and set
*     NROWA, NCOLA and  NROWB  as the number of rows and  columns  of  A
*     and the number of rows of  B  respectively.
*
      NOTA  = LSAME( TRANSA, 'N' )
      NOTB  = LSAME( TRANSB, 'N' )
      CONJA = LSAME( TRANSA, 'C' )
      CONJB = LSAME( TRANSB, 'C' )
      IF( NOTA )THEN
         NROWA = M
         NCOLA = K
      ELSE
         NROWA = K
         NCOLA = M
      END IF
      IF( NOTB )THEN
         NROWB = K
      ELSE
         NROWB = N
      END IF
*
*     Test the input parameters.
*
      INFO = 0
      IF(      ( .NOT.NOTA                 ).AND.
     $         ( .NOT.CONJA                ).AND.
     $         ( .NOT.LSAME( TRANSA, 'T' ) )      )THEN
         INFO = 1
      ELSE IF( ( .NOT.NOTB                 ).AND.
     $         ( .NOT.CONJB                ).AND.
     $         ( .NOT.LSAME( TRANSB, 'T' ) )      )THEN
         INFO = 2
      ELSE IF( M  .LT.0               )THEN
         INFO = 3
      ELSE IF( N  .LT.0               )THEN
         INFO = 4
      ELSE IF( K  .LT.0               )THEN
         INFO = 5
      ELSE IF( LDA.LT.MAX( 1, NROWA ) )THEN
         INFO = 8
      ELSE IF( LDB.LT.MAX( 1, NROWB ) )THEN
         INFO = 10
      ELSE IF( LDC.LT.MAX( 1, M     ) )THEN
         INFO = 13
      END IF
      IF( INFO.NE.0 )THEN
         CALL XERBLA( 'ZGEMM ', INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( ( M.EQ.0 ).OR.( N.EQ.0 ).OR.
     $    ( ( ( ALPHA.EQ.ZERO ).OR.( K.EQ.0 ) ).AND.( BETA.EQ.ONE ) ) )
     $   RETURN
*
*     And when  alpha.eq.zero.
*
      IF( ALPHA.EQ.ZERO )THEN
         IF( BETA.EQ.ZERO )THEN
            DO 20, J = 1, N
               DO 10, I = 1, M
                  C( I, J ) = ZERO
   10          CONTINUE
   20       CONTINUE
         ELSE
            DO 40, J = 1, N
               DO 30, I = 1, M
                  C( I, J ) = BETA*C( I, J )
   30          CONTINUE
   40       CONTINUE
         END IF
         RETURN
      END IF
*
*     Start the operations.
*
      IF( NOTB )THEN
         IF( NOTA )THEN
*
*           Form  C := alpha*A*B + beta*C.
*
            DO 90, J = 1, N
               IF( BETA.EQ.ZERO )THEN
                  DO 50, I = 1, M
                     C( I, J ) = ZERO
   50             CONTINUE
               ELSE IF( BETA.NE.ONE )THEN
                  DO 60, I = 1, M
                     C( I, J ) = BETA*C( I, J )
   60             CONTINUE
               END IF
               DO 80, L = 1, K
                  IF( B( L, J ).NE.ZERO )THEN
                     TEMP = ALPHA*B( L, J )
                     DO 70, I = 1, M
                        C( I, J ) = C( I, J ) + TEMP*A( I, L )
   70                CONTINUE
                  END IF
   80          CONTINUE
   90       CONTINUE
         ELSE IF( CONJA )THEN
*
*           Form  C := alpha*conjg( A' )*B + beta*C.
*
            DO 120, J = 1, N
               DO 110, I = 1, M
                  TEMP = ZERO
                  DO 100, L = 1, K
                     TEMP = TEMP + DCONJG( A( L, I ) )*B( L, J )
  100             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  110          CONTINUE
  120       CONTINUE
         ELSE
*
*           Form  C := alpha*A'*B + beta*C
*
            DO 150, J = 1, N
               DO 140, I = 1, M
                  TEMP = ZERO
                  DO 130, L = 1, K
                     TEMP = TEMP + A( L, I )*B( L, J )
  130             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  140          CONTINUE
  150       CONTINUE
         END IF
      ELSE IF( NOTA )THEN
         IF( CONJB )THEN
*
*           Form  C := alpha*A*conjg( B' ) + beta*C.
*
            DO 200, J = 1, N
               IF( BETA.EQ.ZERO )THEN
                  DO 160, I = 1, M
                     C( I, J ) = ZERO
  160             CONTINUE
               ELSE IF( BETA.NE.ONE )THEN
                  DO 170, I = 1, M
                     C( I, J ) = BETA*C( I, J )
  170             CONTINUE
               END IF
               DO 190, L = 1, K
                  IF( B( J, L ).NE.ZERO )THEN
                     TEMP = ALPHA*DCONJG( B( J, L ) )
                     DO 180, I = 1, M
                        C( I, J ) = C( I, J ) + TEMP*A( I, L )
  180                CONTINUE
                  END IF
  190          CONTINUE
  200       CONTINUE
         ELSE
*
*           Form  C := alpha*A*B'          + beta*C
*
            DO 250, J = 1, N
               IF( BETA.EQ.ZERO )THEN
                  DO 210, I = 1, M
                     C( I, J ) = ZERO
  210             CONTINUE
               ELSE IF( BETA.NE.ONE )THEN
                  DO 220, I = 1, M
                     C( I, J ) = BETA*C( I, J )
  220             CONTINUE
               END IF
               DO 240, L = 1, K
                  IF( B( J, L ).NE.ZERO )THEN
                     TEMP = ALPHA*B( J, L )
                     DO 230, I = 1, M
                        C( I, J ) = C( I, J ) + TEMP*A( I, L )
  230                CONTINUE
                  END IF
  240          CONTINUE
  250       CONTINUE
         END IF
      ELSE IF( CONJA )THEN
         IF( CONJB )THEN
*
*           Form  C := alpha*conjg( A' )*conjg( B' ) + beta*C.
*
            DO 280, J = 1, N
               DO 270, I = 1, M
                  TEMP = ZERO
                  DO 260, L = 1, K
                     TEMP = TEMP +
     $                      DCONJG( A( L, I ) )*DCONJG( B( J, L ) )
  260             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  270          CONTINUE
  280       CONTINUE
         ELSE
*
*           Form  C := alpha*conjg( A' )*B' + beta*C
*
            DO 310, J = 1, N
               DO 300, I = 1, M
                  TEMP = ZERO
                  DO 290, L = 1, K
                     TEMP = TEMP + DCONJG( A( L, I ) )*B( J, L )
  290             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  300          CONTINUE
  310       CONTINUE
         END IF
      ELSE
         IF( CONJB )THEN
*
*           Form  C := alpha*A'*conjg( B' ) + beta*C
*
            DO 340, J = 1, N
               DO 330, I = 1, M
                  TEMP = ZERO
                  DO 320, L = 1, K
                     TEMP = TEMP + A( L, I )*DCONJG( B( J, L ) )
  320             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  330          CONTINUE
  340       CONTINUE
         ELSE
*
*           Form  C := alpha*A'*B' + beta*C
*
            DO 370, J = 1, N
               DO 360, I = 1, M
                  TEMP = ZERO
                  DO 350, L = 1, K
                     TEMP = TEMP + A( L, I )*B( J, L )
  350             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  360          CONTINUE
  370       CONTINUE
         END IF
      END IF
*
      RETURN
*
*     End of ZGEMM .
*
      END
      SUBROUTINE ZGERU ( M, N, ALPHA, X, INCX, Y, INCY, A, LDA )
*     .. Scalar Arguments ..
      COMPLEX*16         ALPHA
      INTEGER            INCX, INCY, LDA, M, N
*     .. Array Arguments ..
      COMPLEX*16         A( LDA, * ), X( * ), Y( * )
*     ..
*
*  Purpose
*  =======
*
*  ZGERU  performs the rank 1 operation
*
*     A := alpha*x*y' + A,
*
*  where alpha is a scalar, x is an m element vector, y is an n element
*  vector and A is an m by n matrix.
*
*  Parameters
*  ==========
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of the matrix A.
*           M must be at least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of the matrix A.
*           N must be at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry, ALPHA specifies the scalar alpha.
*           Unchanged on exit.
*
*  X      - COMPLEX*16       array of dimension at least
*           ( 1 + ( m - 1 )*abs( INCX ) ).
*           Before entry, the incremented array X must contain the m
*           element vector x.
*           Unchanged on exit.
*
*  INCX   - INTEGER.
*           On entry, INCX specifies the increment for the elements of
*           X. INCX must not be zero.
*           Unchanged on exit.
*
*  Y      - COMPLEX*16       array of dimension at least
*           ( 1 + ( n - 1 )*abs( INCY ) ).
*           Before entry, the incremented array Y must contain the n
*           element vector y.
*           Unchanged on exit.
*
*  INCY   - INTEGER.
*           On entry, INCY specifies the increment for the elements of
*           Y. INCY must not be zero.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, n ).
*           Before entry, the leading m by n part of the array A must
*           contain the matrix of coefficients. On exit, A is
*           overwritten by the updated matrix.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program. LDA must be at least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 2 Blas routine.
*
*  -- Written on 22-October-1986.
*     Jack Dongarra, Argonne National Lab.
*     Jeremy Du Croz, Nag Central Office.
*     Sven Hammarling, Nag Central Office.
*     Richard Hanson, Sandia National Labs.
*
*
*     .. Parameters ..
      COMPLEX*16         ZERO
      PARAMETER        ( ZERO = ( 0.0D+0, 0.0D+0 ) )
*     .. Local Scalars ..
      COMPLEX*16         TEMP
      INTEGER            I, INFO, IX, J, JY, KX
*     .. External Subroutines ..
      EXTERNAL           XERBLA
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      IF     ( M.LT.0 )THEN
         INFO = 1
      ELSE IF( N.LT.0 )THEN
         INFO = 2
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 5
      ELSE IF( INCY.EQ.0 )THEN
         INFO = 7
      ELSE IF( LDA.LT.MAX( 1, M ) )THEN
         INFO = 9
      END IF
      IF( INFO.NE.0 )THEN
         CALL XERBLA( 'ZGERU ', INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( ( M.EQ.0 ).OR.( N.EQ.0 ).OR.( ALPHA.EQ.ZERO ) )
     $   RETURN
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
      IF( INCY.GT.0 )THEN
         JY = 1
      ELSE
         JY = 1 - ( N - 1 )*INCY
      END IF
      IF( INCX.EQ.1 )THEN
         DO 20, J = 1, N
            IF( Y( JY ).NE.ZERO )THEN
               TEMP = ALPHA*Y( JY )
               DO 10, I = 1, M
                  A( I, J ) = A( I, J ) + X( I )*TEMP
   10          CONTINUE
            END IF
            JY = JY + INCY
   20    CONTINUE
      ELSE
         IF( INCX.GT.0 )THEN
            KX = 1
         ELSE
            KX = 1 - ( M - 1 )*INCX
         END IF
         DO 40, J = 1, N
            IF( Y( JY ).NE.ZERO )THEN
               TEMP = ALPHA*Y( JY )
               IX   = KX
               DO 30, I = 1, M
                  A( I, J ) = A( I, J ) + X( IX )*TEMP
                  IX        = IX        + INCX
   30          CONTINUE
            END IF
            JY = JY + INCY
   40    CONTINUE
      END IF
*
      RETURN
*
*     End of ZGERU .
*
      END
      SUBROUTINE ZGESV( N, NRHS, A, LDA, IPIV, B, LDB, INFO )
*
*  -- LAPACK driver routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     March 31, 1993
*
*     .. Scalar Arguments ..
      INTEGER            INFO, LDA, LDB, N, NRHS
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      COMPLEX*16         A( LDA, * ), B( LDB, * )
*     ..
*
*  Purpose
*  =======
*
*  ZGESV computes the solution to a complex system of linear equations
*     A * X = B,
*  where A is an N-by-N matrix and X and B are N-by-NRHS matrices.
*
*  The LU decomposition with partial pivoting and row interchanges is
*  used to factor A as
*     A = P * L * U,
*  where P is a permutation matrix, L is unit lower triangular, and U is
*  upper triangular.  The factored form of A is then used to solve the
*  system of equations A * X = B.
*
*  Arguments
*  =========
*
*  N       (input) INTEGER
*          The number of linear equations, i.e., the order of the
*          matrix A.  N >= 0.
*
*  NRHS    (input) INTEGER
*          The number of right hand sides, i.e., the number of columns
*          of the matrix B.  NRHS >= 0.
*
*  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
*          On entry, the N-by-N coefficient matrix A.
*          On exit, the factors L and U from the factorization
*          A = P*L*U; the unit diagonal elements of L are not stored.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,N).
*
*  IPIV    (output) INTEGER array, dimension (N)
*          The pivot indices that define the permutation matrix P;
*          row i of the matrix was interchanged with row IPIV(i).
*
*  B       (input/output) COMPLEX*16 array, dimension (LDB,NRHS)
*          On entry, the N-by-NRHS matrix of right hand side matrix B.
*          On exit, if INFO = 0, the N-by-NRHS solution matrix X.
*
*  LDB     (input) INTEGER
*          The leading dimension of the array B.  LDB >= max(1,N).
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*          > 0:  if INFO = i, U(i,i) is exactly zero.  The factorization
*                has been completed, but the factor U is exactly
*                singular, so the solution could not be computed.
*
*  =====================================================================
*
*     .. External Subroutines ..
      EXTERNAL           XERBLA, ZGETRF, ZGETRS
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      IF( N.LT.0 ) THEN
         INFO = -1
      ELSE IF( NRHS.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -4
      ELSE IF( LDB.LT.MAX( 1, N ) ) THEN
         INFO = -7
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'ZGESV ', -INFO )
         RETURN
      END IF
*
*     Compute the LU factorization of A.
*
      CALL ZGETRF( N, N, A, LDA, IPIV, INFO )
      IF( INFO.EQ.0 ) THEN
*
*        Solve the system A*X = B, overwriting B with X.
*
         CALL ZGETRS( 'No transpose', N, NRHS, A, LDA, IPIV, B, LDB,
     $                INFO )
      END IF
      RETURN
*
*     End of ZGESV
*
      END
      SUBROUTINE ZGETF2( M, N, A, LDA, IPIV, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     September 30, 1994
*
*     .. Scalar Arguments ..
      INTEGER            INFO, LDA, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      COMPLEX*16         A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  ZGETF2 computes an LU factorization of a general m-by-n matrix A
*  using partial pivoting with row interchanges.
*
*  The factorization has the form
*     A = P * L * U
*  where P is a permutation matrix, L is lower triangular with unit
*  diagonal elements (lower trapezoidal if m > n), and U is upper
*  triangular (upper trapezoidal if m < n).
*
*  This is the right-looking Level 2 BLAS version of the algorithm.
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the matrix A.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.  N >= 0.
*
*  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
*          On entry, the m by n matrix to be factored.
*          On exit, the factors L and U from the factorization
*          A = P*L*U; the unit diagonal elements of L are not stored.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,M).
*
*  IPIV    (output) INTEGER array, dimension (min(M,N))
*          The pivot indices; for 1 <= i <= min(M,N), row i of the
*          matrix was interchanged with row IPIV(i).
*
*  INFO    (output) INTEGER
*          = 0: successful exit
*          < 0: if INFO = -k, the k-th argument had an illegal value
*          > 0: if INFO = k, U(k,k) is exactly zero. The factorization
*               has been completed, but the factor U is exactly
*               singular, and division by zero will occur if it is used
*               to solve a system of equations.
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE, ZERO
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ),
     $                   ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      INTEGER            J, JP
*     ..
*     .. External Functions ..
      INTEGER            IZAMAX
      EXTERNAL           IZAMAX
*     ..
*     .. External Subroutines ..
      EXTERNAL           XERBLA, ZGERU, ZSCAL, ZSWAP
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'ZGETF2', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
      DO 10 J = 1, MIN( M, N )
*
*        Find pivot and test for singularity.
*
         JP = J - 1 + IZAMAX( M-J+1, A( J, J ), 1 )
         IPIV( J ) = JP
         IF( A( JP, J ).NE.ZERO ) THEN
*
*           Apply the interchange to columns 1:N.
*
            IF( JP.NE.J )
     $         CALL ZSWAP( N, A( J, 1 ), LDA, A( JP, 1 ), LDA )
*
*           Compute elements J+1:M of J-th column.
*
            IF( J.LT.M )
     $         CALL ZSCAL( M-J, ONE / A( J, J ), A( J+1, J ), 1 )
*
         ELSE IF( INFO.EQ.0 ) THEN
*
            INFO = J
         END IF
*
         IF( J.LT.MIN( M, N ) ) THEN
*
*           Update trailing submatrix.
*
            CALL ZGERU( M-J, N-J, -ONE, A( J+1, J ), 1, A( J, J+1 ),
     $                  LDA, A( J+1, J+1 ), LDA )
         END IF
   10 CONTINUE
      RETURN
*
*     End of ZGETF2
*
      END
      SUBROUTINE ZGETRF( M, N, A, LDA, IPIV, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     September 30, 1994
*
*     .. Scalar Arguments ..
      INTEGER            INFO, LDA, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      COMPLEX*16         A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  ZGETRF computes an LU factorization of a general M-by-N matrix A
*  using partial pivoting with row interchanges.
*
*  The factorization has the form
*     A = P * L * U
*  where P is a permutation matrix, L is lower triangular with unit
*  diagonal elements (lower trapezoidal if m > n), and U is upper
*  triangular (upper trapezoidal if m < n).
*
*  This is the right-looking Level 3 BLAS version of the algorithm.
*
*  Arguments
*  =========
*
*  M       (input) INTEGER
*          The number of rows of the matrix A.  M >= 0.
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.  N >= 0.
*
*  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
*          On entry, the M-by-N matrix to be factored.
*          On exit, the factors L and U from the factorization
*          A = P*L*U; the unit diagonal elements of L are not stored.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,M).
*
*  IPIV    (output) INTEGER array, dimension (min(M,N))
*          The pivot indices; for 1 <= i <= min(M,N), row i of the
*          matrix was interchanged with row IPIV(i).
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*          > 0:  if INFO = i, U(i,i) is exactly zero. The factorization
*                has been completed, but the factor U is exactly
*                singular, and division by zero will occur if it is used
*                to solve a system of equations.
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IINFO, J, JB, NB
*     ..
*     .. External Subroutines ..
      EXTERNAL           XERBLA, ZGEMM, ZGETF2, ZLASWP, ZTRSM
*     ..
*     .. External Functions ..
      INTEGER            ILAENV
      EXTERNAL           ILAENV
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      IF( M.LT.0 ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, M ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'ZGETRF', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
*     Determine the block size for this environment.
*
      NB = ILAENV( 1, 'ZGETRF', ' ', M, N, -1, -1 )
      IF( NB.LE.1 .OR. NB.GE.MIN( M, N ) ) THEN
*
*        Use unblocked code.
*
         CALL ZGETF2( M, N, A, LDA, IPIV, INFO )
      ELSE
*
*        Use blocked code.
*
         DO 20 J = 1, MIN( M, N ), NB
            JB = MIN( MIN( M, N )-J+1, NB )
*
*           Factor diagonal and subdiagonal blocks and test for exact
*           singularity.
*
            CALL ZGETF2( M-J+1, JB, A( J, J ), LDA, IPIV( J ), IINFO )
*
*           Adjust INFO and the pivot indices.
*
            IF( INFO.EQ.0 .AND. IINFO.GT.0 )
     $         INFO = IINFO + J - 1
            DO 10 I = J, MIN( M, J+JB-1 )
               IPIV( I ) = J - 1 + IPIV( I )
   10       CONTINUE
*
*           Apply interchanges to columns 1:J-1.
*
            CALL ZLASWP( J-1, A, LDA, J, J+JB-1, IPIV, 1 )
*
            IF( J+JB.LE.N ) THEN
*
*              Apply interchanges to columns J+JB:N.
*
               CALL ZLASWP( N-J-JB+1, A( 1, J+JB ), LDA, J, J+JB-1,
     $                      IPIV, 1 )
*
*              Compute block row of U.
*
               CALL ZTRSM( 'Left', 'Lower', 'No transpose', 'Unit', JB,
     $                     N-J-JB+1, ONE, A( J, J ), LDA, A( J, J+JB ),
     $                     LDA )
               IF( J+JB.LE.M ) THEN
*
*                 Update trailing submatrix.
*
                  CALL ZGEMM( 'No transpose', 'No transpose', M-J-JB+1,
     $                        N-J-JB+1, JB, -ONE, A( J+JB, J ), LDA,
     $                        A( J, J+JB ), LDA, ONE, A( J+JB, J+JB ),
     $                        LDA )
               END IF
            END IF
   20    CONTINUE
      END IF
      RETURN
*
*     End of ZGETRF
*
      END
      SUBROUTINE ZGETRS( TRANS, N, NRHS, A, LDA, IPIV, B, LDB, INFO )
*
*  -- LAPACK routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     September 30, 1994
*
*     .. Scalar Arguments ..
      CHARACTER          TRANS
      INTEGER            INFO, LDA, LDB, N, NRHS
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      COMPLEX*16         A( LDA, * ), B( LDB, * )
*     ..
*
*  Purpose
*  =======
*
*  ZGETRS solves a system of linear equations
*     A * X = B,  A**T * X = B,  or  A**H * X = B
*  with a general N-by-N matrix A using the LU factorization computed
*  by ZGETRF.
*
*  Arguments
*  =========
*
*  TRANS   (input) CHARACTER*1
*          Specifies the form of the system of equations:
*          = 'N':  A * X = B     (No transpose)
*          = 'T':  A**T * X = B  (Transpose)
*          = 'C':  A**H * X = B  (Conjugate transpose)
*
*  N       (input) INTEGER
*          The order of the matrix A.  N >= 0.
*
*  NRHS    (input) INTEGER
*          The number of right hand sides, i.e., the number of columns
*          of the matrix B.  NRHS >= 0.
*
*  A       (input) COMPLEX*16 array, dimension (LDA,N)
*          The factors L and U from the factorization A = P*L*U
*          as computed by ZGETRF.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.  LDA >= max(1,N).
*
*  IPIV    (input) INTEGER array, dimension (N)
*          The pivot indices from ZGETRF; for 1<=i<=N, row i of the
*          matrix was interchanged with row IPIV(i).
*
*  B       (input/output) COMPLEX*16 array, dimension (LDB,NRHS)
*          On entry, the right hand side matrix B.
*          On exit, the solution matrix X.
*
*  LDB     (input) INTEGER
*          The leading dimension of the array B.  LDB >= max(1,N).
*
*  INFO    (output) INTEGER
*          = 0:  successful exit
*          < 0:  if INFO = -i, the i-th argument had an illegal value
*
*  =====================================================================
*
*     .. Parameters ..
      COMPLEX*16         ONE
      PARAMETER          ( ONE = ( 1.0D+0, 0.0D+0 ) )
*     ..
*     .. Local Scalars ..
      LOGICAL            NOTRAN
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. External Subroutines ..
      EXTERNAL           XERBLA, ZLASWP, ZTRSM
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      INFO = 0
      NOTRAN = LSAME( TRANS, 'N' )
      IF( .NOT.NOTRAN .AND. .NOT.LSAME( TRANS, 'T' ) .AND. .NOT.
     $    LSAME( TRANS, 'C' ) ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( NRHS.LT.0 ) THEN
         INFO = -3
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -5
      ELSE IF( LDB.LT.MAX( 1, N ) ) THEN
         INFO = -8
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'ZGETRS', -INFO )
         RETURN
      END IF
*
*     Quick return if possible
*
      IF( N.EQ.0 .OR. NRHS.EQ.0 )
     $   RETURN
*
      IF( NOTRAN ) THEN
*
*        Solve A * X = B.
*
*        Apply row interchanges to the right hand sides.
*
         CALL ZLASWP( NRHS, B, LDB, 1, N, IPIV, 1 )
*
*        Solve L*X = B, overwriting B with X.
*
         CALL ZTRSM( 'Left', 'Lower', 'No transpose', 'Unit', N, NRHS,
     $               ONE, A, LDA, B, LDB )
*
*        Solve U*X = B, overwriting B with X.
*
         CALL ZTRSM( 'Left', 'Upper', 'No transpose', 'Non-unit', N,
     $               NRHS, ONE, A, LDA, B, LDB )
      ELSE
*
*        Solve A**T * X = B  or A**H * X = B.
*
*        Solve U'*X = B, overwriting B with X.
*
         CALL ZTRSM( 'Left', 'Upper', TRANS, 'Non-unit', N, NRHS, ONE,
     $               A, LDA, B, LDB )
*
*        Solve L'*X = B, overwriting B with X.
*
         CALL ZTRSM( 'Left', 'Lower', TRANS, 'Unit', N, NRHS, ONE, A,
     $               LDA, B, LDB )
*
*        Apply row interchanges to the solution vectors.
*
         CALL ZLASWP( NRHS, B, LDB, 1, N, IPIV, -1 )
      END IF
*
      RETURN
*
*     End of ZGETRS
*
      END
      SUBROUTINE ZLASWP( N, A, LDA, K1, K2, IPIV, INCX )
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     June 30, 1999
*
*     .. Scalar Arguments ..
      INTEGER            INCX, K1, K2, LDA, N
*     ..
*     .. Array Arguments ..
      INTEGER            IPIV( * )
      COMPLEX*16         A( LDA, * )
*     ..
*
*  Purpose
*  =======
*
*  ZLASWP performs a series of row interchanges on the matrix A.
*  One row interchange is initiated for each of rows K1 through K2 of A.
*
*  Arguments
*  =========
*
*  N       (input) INTEGER
*          The number of columns of the matrix A.
*
*  A       (input/output) COMPLEX*16 array, dimension (LDA,N)
*          On entry, the matrix of column dimension N to which the row
*          interchanges will be applied.
*          On exit, the permuted matrix.
*
*  LDA     (input) INTEGER
*          The leading dimension of the array A.
*
*  K1      (input) INTEGER
*          The first element of IPIV for which a row interchange will
*          be done.
*
*  K2      (input) INTEGER
*          The last element of IPIV for which a row interchange will
*          be done.
*
*  IPIV    (input) INTEGER array, dimension (M*abs(INCX))
*          The vector of pivot indices.  Only the elements in positions
*          K1 through K2 of IPIV are accessed.
*          IPIV(K) = L implies rows K and L are to be interchanged.
*
*  INCX    (input) INTEGER
*          The increment between successive values of IPIV.  If IPIV
*          is negative, the pivots are applied in reverse order.
*
*  Further Details
*  ===============
*
*  Modified by
*   R. C. Whaley, Computer Science Dept., Univ. of Tenn., Knoxville, USA
*
* =====================================================================
*
*     .. Local Scalars ..
      INTEGER            I, I1, I2, INC, IP, IX, IX0, J, K, N32
      COMPLEX*16         TEMP
*     ..
*     .. Executable Statements ..
*
*     Interchange row I with row IPIV(I) for each of rows K1 through K2.
*
      IF( INCX.GT.0 ) THEN
         IX0 = K1
         I1 = K1
         I2 = K2
         INC = 1
      ELSE IF( INCX.LT.0 ) THEN
         IX0 = 1 + ( 1-K2 )*INCX
         I1 = K2
         I2 = K1
         INC = -1
      ELSE
         RETURN
      END IF
*
      N32 = ( N / 32 )*32
      IF( N32.NE.0 ) THEN
         DO 30 J = 1, N32, 32
            IX = IX0
            DO 20 I = I1, I2, INC
               IP = IPIV( IX )
               IF( IP.NE.I ) THEN
                  DO 10 K = J, J + 31
                     TEMP = A( I, K )
                     A( I, K ) = A( IP, K )
                     A( IP, K ) = TEMP
   10             CONTINUE
               END IF
               IX = IX + INCX
   20       CONTINUE
   30    CONTINUE
      END IF
      IF( N32.NE.N ) THEN
         N32 = N32 + 1
         IX = IX0
         DO 50 I = I1, I2, INC
            IP = IPIV( IX )
            IF( IP.NE.I ) THEN
               DO 40 K = N32, N
                  TEMP = A( I, K )
                  A( I, K ) = A( IP, K )
                  A( IP, K ) = TEMP
   40          CONTINUE
            END IF
            IX = IX + INCX
   50    CONTINUE
      END IF
*
      RETURN
*
*     End of ZLASWP
*
      END
      subroutine  zscal(n,za,zx,incx)
c
c     scales a vector by a constant.
c     jack dongarra, 3/11/78.
c     modified 3/93 to return if incx .le. 0.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double complex za,zx(*)
      integer i,incx,ix,n
c
      if( n.le.0 .or. incx.le.0 )return
      if(incx.eq.1)go to 20
c
c        code for increment not equal to 1
c
      ix = 1
      do 10 i = 1,n
        zx(ix) = za*zx(ix)
        ix = ix + incx
   10 continue
      return
c
c        code for increment equal to 1
c
   20 do 30 i = 1,n
        zx(i) = za*zx(i)
   30 continue
      return
      end
      subroutine  zswap (n,zx,incx,zy,incy)
c
c     interchanges two vectors.
c     jack dongarra, 3/11/78.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double complex zx(*),zy(*),ztemp
      integer i,incx,incy,ix,iy,n
c
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c       code for unequal increments or equal increments not equal
c         to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        ztemp = zx(ix)
        zx(ix) = zy(iy)
        zy(iy) = ztemp
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c       code for both increments equal to 1
   20 do 30 i = 1,n
        ztemp = zx(i)
        zx(i) = zy(i)
        zy(i) = ztemp
   30 continue
      return
      end
      SUBROUTINE ZTRSM ( SIDE, UPLO, TRANSA, DIAG, M, N, ALPHA, A, LDA,
     $                   B, LDB )
*     .. Scalar Arguments ..
      CHARACTER*1        SIDE, UPLO, TRANSA, DIAG
      INTEGER            M, N, LDA, LDB
      COMPLEX*16         ALPHA
*     .. Array Arguments ..
      COMPLEX*16         A( LDA, * ), B( LDB, * )
*     ..
*
*  Purpose
*  =======
*
*  ZTRSM  solves one of the matrix equations
*
*     op( A )*X = alpha*B,   or   X*op( A ) = alpha*B,
*
*  where alpha is a scalar, X and B are m by n matrices, A is a unit, or
*  non-unit,  upper or lower triangular matrix  and  op( A )  is one  of
*
*     op( A ) = A   or   op( A ) = A'   or   op( A ) = conjg( A' ).
*
*  The matrix X is overwritten on B.
*
*  Parameters
*  ==========
*
*  SIDE   - CHARACTER*1.
*           On entry, SIDE specifies whether op( A ) appears on the left
*           or right of X as follows:
*
*              SIDE = 'L' or 'l'   op( A )*X = alpha*B.
*
*              SIDE = 'R' or 'r'   X*op( A ) = alpha*B.
*
*           Unchanged on exit.
*
*  UPLO   - CHARACTER*1.
*           On entry, UPLO specifies whether the matrix A is an upper or
*           lower triangular matrix as follows:
*
*              UPLO = 'U' or 'u'   A is an upper triangular matrix.
*
*              UPLO = 'L' or 'l'   A is a lower triangular matrix.
*
*           Unchanged on exit.
*
*  TRANSA - CHARACTER*1.
*           On entry, TRANSA specifies the form of op( A ) to be used in
*           the matrix multiplication as follows:
*
*              TRANSA = 'N' or 'n'   op( A ) = A.
*
*              TRANSA = 'T' or 't'   op( A ) = A'.
*
*              TRANSA = 'C' or 'c'   op( A ) = conjg( A' ).
*
*           Unchanged on exit.
*
*  DIAG   - CHARACTER*1.
*           On entry, DIAG specifies whether or not A is unit triangular
*           as follows:
*
*              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
*
*              DIAG = 'N' or 'n'   A is not assumed to be unit
*                                  triangular.
*
*           Unchanged on exit.
*
*  M      - INTEGER.
*           On entry, M specifies the number of rows of B. M must be at
*           least zero.
*           Unchanged on exit.
*
*  N      - INTEGER.
*           On entry, N specifies the number of columns of B.  N must be
*           at least zero.
*           Unchanged on exit.
*
*  ALPHA  - COMPLEX*16      .
*           On entry,  ALPHA specifies the scalar  alpha. When  alpha is
*           zero then  A is not referenced and  B need not be set before
*           entry.
*           Unchanged on exit.
*
*  A      - COMPLEX*16       array of DIMENSION ( LDA, k ), where k is m
*           when  SIDE = 'L' or 'l'  and is  n  when  SIDE = 'R' or 'r'.
*           Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
*           upper triangular part of the array  A must contain the upper
*           triangular matrix  and the strictly lower triangular part of
*           A is not referenced.
*           Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
*           lower triangular part of the array  A must contain the lower
*           triangular matrix  and the strictly upper triangular part of
*           A is not referenced.
*           Note that when  DIAG = 'U' or 'u',  the diagonal elements of
*           A  are not referenced either,  but are assumed to be  unity.
*           Unchanged on exit.
*
*  LDA    - INTEGER.
*           On entry, LDA specifies the first dimension of A as declared
*           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
*           LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
*           then LDA must be at least max( 1, n ).
*           Unchanged on exit.
*
*  B      - COMPLEX*16       array of DIMENSION ( LDB, n ).
*           Before entry,  the leading  m by n part of the array  B must
*           contain  the  right-hand  side  matrix  B,  and  on exit  is
*           overwritten by the solution matrix  X.
*
*  LDB    - INTEGER.
*           On entry, LDB specifies the first dimension of B as declared
*           in  the  calling  (sub)  program.   LDB  must  be  at  least
*           max( 1, m ).
*           Unchanged on exit.
*
*
*  Level 3 Blas routine.
*
*  -- Written on 8-February-1989.
*     Jack Dongarra, Argonne National Laboratory.
*     Iain Duff, AERE Harwell.
*     Jeremy Du Croz, Numerical Algorithms Group Ltd.
*     Sven Hammarling, Numerical Algorithms Group Ltd.
*
*
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     .. External Subroutines ..
      EXTERNAL           XERBLA
*     .. Intrinsic Functions ..
      INTRINSIC          DCONJG, MAX
*     .. Local Scalars ..
      LOGICAL            LSIDE, NOCONJ, NOUNIT, UPPER
      INTEGER            I, INFO, J, K, NROWA
      COMPLEX*16         TEMP
*     .. Parameters ..
      COMPLEX*16         ONE
      PARAMETER        ( ONE  = ( 1.0D+0, 0.0D+0 ) )
      COMPLEX*16         ZERO
      PARAMETER        ( ZERO = ( 0.0D+0, 0.0D+0 ) )
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
      LSIDE  = LSAME( SIDE  , 'L' )
      IF( LSIDE )THEN
         NROWA = M
      ELSE
         NROWA = N
      END IF
      NOCONJ = LSAME( TRANSA, 'T' )
      NOUNIT = LSAME( DIAG  , 'N' )
      UPPER  = LSAME( UPLO  , 'U' )
*
      INFO   = 0
      IF(      ( .NOT.LSIDE                ).AND.
     $         ( .NOT.LSAME( SIDE  , 'R' ) )      )THEN
         INFO = 1
      ELSE IF( ( .NOT.UPPER                ).AND.
     $         ( .NOT.LSAME( UPLO  , 'L' ) )      )THEN
         INFO = 2
      ELSE IF( ( .NOT.LSAME( TRANSA, 'N' ) ).AND.
     $         ( .NOT.LSAME( TRANSA, 'T' ) ).AND.
     $         ( .NOT.LSAME( TRANSA, 'C' ) )      )THEN
         INFO = 3
      ELSE IF( ( .NOT.LSAME( DIAG  , 'U' ) ).AND.
     $         ( .NOT.LSAME( DIAG  , 'N' ) )      )THEN
         INFO = 4
      ELSE IF( M  .LT.0               )THEN
         INFO = 5
      ELSE IF( N  .LT.0               )THEN
         INFO = 6
      ELSE IF( LDA.LT.MAX( 1, NROWA ) )THEN
         INFO = 9
      ELSE IF( LDB.LT.MAX( 1, M     ) )THEN
         INFO = 11
      END IF
      IF( INFO.NE.0 )THEN
         CALL XERBLA( 'ZTRSM ', INFO )
         RETURN
      END IF
*
*     Quick return if possible.
*
      IF( N.EQ.0 )
     $   RETURN
*
*     And when  alpha.eq.zero.
*
      IF( ALPHA.EQ.ZERO )THEN
         DO 20, J = 1, N
            DO 10, I = 1, M
               B( I, J ) = ZERO
   10       CONTINUE
   20    CONTINUE
         RETURN
      END IF
*
*     Start the operations.
*
      IF( LSIDE )THEN
         IF( LSAME( TRANSA, 'N' ) )THEN
*
*           Form  B := alpha*inv( A )*B.
*
            IF( UPPER )THEN
               DO 60, J = 1, N
                  IF( ALPHA.NE.ONE )THEN
                     DO 30, I = 1, M
                        B( I, J ) = ALPHA*B( I, J )
   30                CONTINUE
                  END IF
                  DO 50, K = M, 1, -1
                     IF( B( K, J ).NE.ZERO )THEN
                        IF( NOUNIT )
     $                     B( K, J ) = B( K, J )/A( K, K )
                        DO 40, I = 1, K - 1
                           B( I, J ) = B( I, J ) - B( K, J )*A( I, K )
   40                   CONTINUE
                     END IF
   50             CONTINUE
   60          CONTINUE
            ELSE
               DO 100, J = 1, N
                  IF( ALPHA.NE.ONE )THEN
                     DO 70, I = 1, M
                        B( I, J ) = ALPHA*B( I, J )
   70                CONTINUE
                  END IF
                  DO 90 K = 1, M
                     IF( B( K, J ).NE.ZERO )THEN
                        IF( NOUNIT )
     $                     B( K, J ) = B( K, J )/A( K, K )
                        DO 80, I = K + 1, M
                           B( I, J ) = B( I, J ) - B( K, J )*A( I, K )
   80                   CONTINUE
                     END IF
   90             CONTINUE
  100          CONTINUE
            END IF
         ELSE
*
*           Form  B := alpha*inv( A' )*B
*           or    B := alpha*inv( conjg( A' ) )*B.
*
            IF( UPPER )THEN
               DO 140, J = 1, N
                  DO 130, I = 1, M
                     TEMP = ALPHA*B( I, J )
                     IF( NOCONJ )THEN
                        DO 110, K = 1, I - 1
                           TEMP = TEMP - A( K, I )*B( K, J )
  110                   CONTINUE
                        IF( NOUNIT )
     $                     TEMP = TEMP/A( I, I )
                     ELSE
                        DO 120, K = 1, I - 1
                           TEMP = TEMP - DCONJG( A( K, I ) )*B( K, J )
  120                   CONTINUE
                        IF( NOUNIT )
     $                     TEMP = TEMP/DCONJG( A( I, I ) )
                     END IF
                     B( I, J ) = TEMP
  130             CONTINUE
  140          CONTINUE
            ELSE
               DO 180, J = 1, N
                  DO 170, I = M, 1, -1
                     TEMP = ALPHA*B( I, J )
                     IF( NOCONJ )THEN
                        DO 150, K = I + 1, M
                           TEMP = TEMP - A( K, I )*B( K, J )
  150                   CONTINUE
                        IF( NOUNIT )
     $                     TEMP = TEMP/A( I, I )
                     ELSE
                        DO 160, K = I + 1, M
                           TEMP = TEMP - DCONJG( A( K, I ) )*B( K, J )
  160                   CONTINUE
                        IF( NOUNIT )
     $                     TEMP = TEMP/DCONJG( A( I, I ) )
                     END IF
                     B( I, J ) = TEMP
  170             CONTINUE
  180          CONTINUE
            END IF
         END IF
      ELSE
         IF( LSAME( TRANSA, 'N' ) )THEN
*
*           Form  B := alpha*B*inv( A ).
*
            IF( UPPER )THEN
               DO 230, J = 1, N
                  IF( ALPHA.NE.ONE )THEN
                     DO 190, I = 1, M
                        B( I, J ) = ALPHA*B( I, J )
  190                CONTINUE
                  END IF
                  DO 210, K = 1, J - 1
                     IF( A( K, J ).NE.ZERO )THEN
                        DO 200, I = 1, M
                           B( I, J ) = B( I, J ) - A( K, J )*B( I, K )
  200                   CONTINUE
                     END IF
  210             CONTINUE
                  IF( NOUNIT )THEN
                     TEMP = ONE/A( J, J )
                     DO 220, I = 1, M
                        B( I, J ) = TEMP*B( I, J )
  220                CONTINUE
                  END IF
  230          CONTINUE
            ELSE
               DO 280, J = N, 1, -1
                  IF( ALPHA.NE.ONE )THEN
                     DO 240, I = 1, M
                        B( I, J ) = ALPHA*B( I, J )
  240                CONTINUE
                  END IF
                  DO 260, K = J + 1, N
                     IF( A( K, J ).NE.ZERO )THEN
                        DO 250, I = 1, M
                           B( I, J ) = B( I, J ) - A( K, J )*B( I, K )
  250                   CONTINUE
                     END IF
  260             CONTINUE
                  IF( NOUNIT )THEN
                     TEMP = ONE/A( J, J )
                     DO 270, I = 1, M
                       B( I, J ) = TEMP*B( I, J )
  270                CONTINUE
                  END IF
  280          CONTINUE
            END IF
         ELSE
*
*           Form  B := alpha*B*inv( A' )
*           or    B := alpha*B*inv( conjg( A' ) ).
*
            IF( UPPER )THEN
               DO 330, K = N, 1, -1
                  IF( NOUNIT )THEN
                     IF( NOCONJ )THEN
                        TEMP = ONE/A( K, K )
                     ELSE
                        TEMP = ONE/DCONJG( A( K, K ) )
                     END IF
                     DO 290, I = 1, M
                        B( I, K ) = TEMP*B( I, K )
  290                CONTINUE
                  END IF
                  DO 310, J = 1, K - 1
                     IF( A( J, K ).NE.ZERO )THEN
                        IF( NOCONJ )THEN
                           TEMP = A( J, K )
                        ELSE
                           TEMP = DCONJG( A( J, K ) )
                        END IF
                        DO 300, I = 1, M
                           B( I, J ) = B( I, J ) - TEMP*B( I, K )
  300                   CONTINUE
                     END IF
  310             CONTINUE
                  IF( ALPHA.NE.ONE )THEN
                     DO 320, I = 1, M
                        B( I, K ) = ALPHA*B( I, K )
  320                CONTINUE
                  END IF
  330          CONTINUE
            ELSE
               DO 380, K = 1, N
                  IF( NOUNIT )THEN
                     IF( NOCONJ )THEN
                        TEMP = ONE/A( K, K )
                     ELSE
                        TEMP = ONE/DCONJG( A( K, K ) )
                     END IF
                     DO 340, I = 1, M
                        B( I, K ) = TEMP*B( I, K )
  340                CONTINUE
                  END IF
                  DO 360, J = K + 1, N
                     IF( A( J, K ).NE.ZERO )THEN
                        IF( NOCONJ )THEN
                           TEMP = A( J, K )
                        ELSE
                           TEMP = DCONJG( A( J, K ) )
                        END IF
                        DO 350, I = 1, M
                           B( I, J ) = B( I, J ) - TEMP*B( I, K )
  350                   CONTINUE
                     END IF
  360             CONTINUE
                  IF( ALPHA.NE.ONE )THEN
                     DO 370, I = 1, M
                        B( I, K ) = ALPHA*B( I, K )
  370                CONTINUE
                  END IF
  380          CONTINUE
            END IF
         END IF
      END IF
*
      RETURN
*
*     End of ZTRSM .
*
      END
      LOGICAL          FUNCTION LSAME( CA, CB )
*
*  -- LAPACK auxiliary routine (version 3.0) --
*     Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
*     Courant Institute, Argonne National Lab, and Rice University
*     September 30, 1994
*
*     .. Scalar Arguments ..
      CHARACTER          CA, CB
*     ..
*
*  Purpose
*  =======
*
*  LSAME returns .TRUE. if CA is the same letter as CB regardless of
*  case.
*
*  Arguments
*  =========
*
*  CA      (input) CHARACTER*1
*  CB      (input) CHARACTER*1
*          CA and CB specify the single characters to be compared.
*
* =====================================================================
*
*     .. Intrinsic Functions ..
      INTRINSIC          ICHAR
*     ..
*     .. Local Scalars ..
      INTEGER            INTA, INTB, ZCODE
*     ..
*     .. Executable Statements ..
*
*     Test if the characters are equal
*
      LSAME = CA.EQ.CB
      IF( LSAME )
     $   RETURN
*
*     Now test for equivalence if both characters are alphabetic.
*
      ZCODE = ICHAR( 'Z' )
*
*     Use 'Z' rather than 'A' so that ASCII can be detected on Prime
*     machines, on which ICHAR returns a value with bit 8 set.
*     ICHAR('A') on Prime machines returns 193 which is the same as
*     ICHAR('A') on an EBCDIC machine.
*
      INTA = ICHAR( CA )
      INTB = ICHAR( CB )
*
      IF( ZCODE.EQ.90 .OR. ZCODE.EQ.122 ) THEN
*
*        ASCII is assumed - ZCODE is the ASCII code of either lower or
*        upper case 'Z'.
*
         IF( INTA.GE.97 .AND. INTA.LE.122 ) INTA = INTA - 32
         IF( INTB.GE.97 .AND. INTB.LE.122 ) INTB = INTB - 32
*
      ELSE IF( ZCODE.EQ.233 .OR. ZCODE.EQ.169 ) THEN
*
*        EBCDIC is assumed - ZCODE is the EBCDIC code of either lower or
*        upper case 'Z'.
*
         IF( INTA.GE.129 .AND. INTA.LE.137 .OR.
     $       INTA.GE.145 .AND. INTA.LE.153 .OR.
     $       INTA.GE.162 .AND. INTA.LE.169 ) INTA = INTA + 64
         IF( INTB.GE.129 .AND. INTB.LE.137 .OR.
     $       INTB.GE.145 .AND. INTB.LE.153 .OR.
     $       INTB.GE.162 .AND. INTB.LE.169 ) INTB = INTB + 64
*
      ELSE IF( ZCODE.EQ.218 .OR. ZCODE.EQ.250 ) THEN
*
*        ASCII is assumed, on Prime machines - ZCODE is the ASCII code
*        plus 128 of either lower or upper case 'Z'.
*
         IF( INTA.GE.225 .AND. INTA.LE.250 ) INTA = INTA - 32
         IF( INTB.GE.225 .AND. INTB.LE.250 ) INTB = INTB - 32
      END IF
      LSAME = INTA.EQ.INTB
*
*     RETURN
*
*     End of LSAME
*
      END
