c     subroutine [q,v,x,y,w,p,z] = lint1(x0,y1,y2,c,L,P,N,ee)

      implicit double precision ( a - h, o - y )
      implicit double complex ( z )
      parameter( mx = 20, my = 1000, mxy = mx * my )
      parameter( pi = 3.1415926535898d0 )
      dimension zg( mxy ), zz( mxy ), zd( mxy ), ad( mxy )
      dimension ixmin( my ), zmin( my ), num( 2 )

c     This function finds contours which zigzag left and right 
c     around zeros of the zeta 
c     function in the strip 
c     x0-xzig<Re s<x0+xzig, boxstart<Im s<boxstart + boxheight, 
c     and then computes
c     (1/2*pi*i) times the integral of the log derivative of zeta
c     along the contours.
c     Output:
c     fort.10	gx	Grid point x-coordinate 1:nz
c     fort.11	gy	Grid point y-coordinate 1:nz
c     fort.12	ad	Absolute value of log derivative at grid pt 1:nz
c     fort.13	px	Path x 1:ny
c     fort.14	py	Path y 1:ny
c     fort.15	lb	Lower bound 1:ny
c     fort.16	ub	Upper bound 1:ny

      cc = -40.0d0
      nl = 5
      np = 2
      nx = 5
      x0 = -0.5d0
      xzig = 0.1d0
      boxstart = 260.0d0
      boxheight = 10.0d0
      nbox = 1

      write( 6, *) ' enter cc, nl, np, nx, x0, xzig'
      read( 5, *, err=2000, end=2000) cc, nl, np, nx, x0, xzig
      write( 6, *) ' cc, nl, np, nx, x0, xzig = ',
     >      cc, nl, np, nx, x0, xzig
      
      write( 6, * ) ' boxstart, boxheight, nbox '
      read( 5, *, err=2000, end=2000 ) boxstart, boxheight, nbox
      write( 6, * ) ' boxstart, boxheight, nbox = ',
     >      boxstart, boxheight, nbox

      iogx = 1
      iogy = 1
      ioad = 1
      iopx = 1
      iopy = 1
      iolb = 1
      ioub = 1

      write( 6, * ) ' iogx, iogy, ioad, iopx, iopy, iolb, ioub '
      read( 5, *, err=2000, end=2000 ) 
     >      iogx, iogy, ioad, iopx, iopy, iolb, ioub
      write( 6, * ) ' iogx, iogy, ioad, iopx, iopy, iolb, ioub = ',
     >      iogx, iogy, ioad, iopx, iopy, iolb, ioub

c     The factor 1.5 ensures that left and right contours do not cross.
      xleft = x0 - 1.5d0 * xzig
      xright = x0 + 1.5d0 * xzig

1000  format( 1(1x, g18.12) )

      do ibox = 1, nbox

c     In each box, we zigzag left (1) and right (2) to get 
c     upper and lower bounds.
      do ixbox = 1, 2
      if( ixbox .eq. 1 ) then
         x1 = xright - xzig
         x2 = xright + xzig
      else
         x1 = xleft - xzig
         x2 = xleft + xzig
      end if
      hx = ( x2 - x1 ) / max( 1, nx - 1 )
 
      y1 = boxstart + ( ibox - 1) * boxheight
      y2 = y1 + boxheight
      ny = max( 2, nint( ( y2 - y1 ) / hx ) )
      hy = ( y2 - y1 ) / max( 1, ny - 1 )

      nz = nx * ny
      do iy = 1, ny
         yy = y1 + ( iy - 1 ) * hy
         do ix = 1, nx
            xx = x1 + ( ix - 1 ) * hx
            zg( ( iy - 1 ) * nx + ix ) = dcmplx( xx, yy )
         end do
      end do
      if( iogx .ne. 0 ) then
         write( 10, 1000 ) ( 
     >     dble( zg( iz ) ), 
     >      iz = 1, nz )
      end if
      if( iogy .ne. 0 ) then
         write( 11, 1000 ) ( 
     >      imag( zg( iz ) ), 
     >      iz = 1, nz )
      end if

      call zeta( nz, zg, zz, zd, cc, nl, np )

      do ixy = 1, nz
        ad( ixy ) = abs( zd( ixy ) )
      end do
      if( ioad .ne. 0 ) then
         write( 12, 1000 ) ( ad( iz ), iz = 1, nz )
      end if

      do iy = 1, ny
        ixbeg = ( iy - 1 ) * nx

        ixm = 1
        tmin = ad( ixbeg + 1 )
        do ix = 2, nx
           adix = ad( ixbeg + ix )
           if( adix .lt. tmin ) then
              tmin = adix
              ixm = ix
           end if
        end do
        ixmin( iy ) = ixm
      end do

      if( iopx .ne. 0 ) then
         write( 13, 1000 ) ( 
     >    dble( zg( ( iy - 1 ) * nx + ixmin( iy ) ) ), 
     >    iy = 1, ny )
      end if

      if( iopy .ne. 0 ) then
         write( 14, 1000 ) ( 
     >    imag( zg( ( iy - 1 ) * nx + ixmin( iy ) ) ), 
     >    iy = 1, ny )
      end if

      zsum = 0.0d0
      do iy = 1, ny - 1
        zvy = zz( ( iy - 1 ) * nx + ixmin( iy ) )
        ip = iy + 1
        zvp = zz( ( ip - 1 ) * nx + ixmin( ip ) )
        zterm = log( zvp / zvy )
        zsum = zsum + zterm
      end do
      zsum = 0.50d0 * dcmplx( 0.0d0, 1.0d0 ) * zsum / pi 
      num( ixbox ) = nint( dble( zsum ) )

      end do
      
      write( 6, * ) ' y1, y2, num = ', y1, y2, num( 1 ), num( 2 )
      if( iolb .ne. 0 ) then
         write(15, *) num( 1 )
      end if
      if( ioub .ne. 0 ) then
         write(16, *) num( 2 )
      end if

      end do

2000  continue

      end
