function zz = plot_surf_zlogd( xl, xr, yl, yr, nx, ny, c, L, P )
% function zz = plot_surf_zlogd( xl, xr, yl, yr, nx, ny, c, L, P )
% Plots surface of logarithmic derivative of zeta( c ) where c is parameter
% in map over rectangle [xl,xr] X [yl,yr] with nx by ny grid. Adds zigzags
% produced by lint.x and saves in PostScript plot file.

 % Set useful input values for unspecified arguments.

 if( nargin < 1 )
  xl = -2.0;
 end
 if( nargin < 2 )
  xr = 0.0;
 end
 if( nargin < 3 )
  yl = 1970.0;
 end
 if( nargin < 4 )
  yr = 1990.0;
 end
 if( nargin < 5 )
  nx = 40;
 end
 if( nargin < 6 )
  ny = 200;
 end
 % Parameter c in z^2 + c map.
 if( nargin < 7 )
  c = -10240;
 end

 % Number of levels to iterate around Julia set.
 if( nargin < 8 )
  L = 6;
 end

 % Number of powers per interval = block size of transfer matrix.
 if( nargin < 9 )
  P = 2;
 end

 % Loop through plotting twice, once on large square (top row)
 % and once on square half the size in each dimension.
 for ibox = 1 : 2

  clf;
  colormap('default');
  brighten( 0.5 );

 if( ibox == 1 )
  xil = xl + 0.5 * ( xr - xl ) - ( xr - xl );
  xir = xr - 0.5 * ( xr - xl ) + ( xr - xl );
  yil = yl + 0.5 * ( yr - yl ) - ( yr - yl );
  yir = yr - 0.5 * ( yr - yl ) + ( yr - yl );
 end
 if( ibox == 2 )
  xil = xl;
  xir = xr;
  yil = yl;
  yir = yr;
 end
 
 % Build grid for zlogd evaluation.
 for kx = 1 : nx
  xx( kx ) = xil + ( kx - 1 ) * ( xir - xil ) / ( nx - 1 );
 end
 for ky = 1 : ny
  yy( ky ) = yil + ( ky - 1 ) * ( yir - yil ) / ( ny - 1 );
 end
 for kx = 1 : nx
 for ky = 1 : ny
  sv( ky, kx ) = xx( kx ) + sqrt( -1.0 ) * yy( ky );
 end
 end
 nz = nx * ny;

 verbose = 0;
 zz = zlogd( sv, c, L, P, verbose );

 % Chop off top few percent of values to make graph look nicer.
 zr = sort( reshape( abs( zz ), nz, 1 ) );

 tfrac = min( nz, max( 1, round( 0.99 * nz ) ) );
 zmax = zr( tfrac );
 zmin = zr( 1 );
 for kx = 1 : nx
 for ky = 1 : ny
  zc( ky, kx ) = min( zmax, max( zmin, abs( zz( ky, kx ) ) ) );
 end 
 end
 caxis( [ zmin, zmax ] );

 subplot( 1, 2, 1 );
% axis( [ xil xir yil yir zmin zmax ] );
 surfc( xx, yy, zc );
% axis( [ xil xir yil yir zmin zmax ] );
% Interpolated shading seems to produce nicer pictures,
% but they take 400MB file size!!! Try with coarser grid?
 shading interp;
% axis fill;

 subplot( 1, 2, 2 );
 nc = 20;
% axis( [ xil xir yil yir ] );
 contour( xx, yy, zc, nc ); colorbar;
% axis( [ xil xir yil yir ] );
% axis fill;
                                                                      
 fn0 = sprintf( 'zlogd/c%dL%dP%dyl%dyr%dnx%dny%d', round( c ), L, P, round( yil ), round( yir ), nx, ny );
 print('-depsc', '-r300', fn0 )

 hold on;

 % Now call external Fortran code lint.x to store zigzags in fort.1[3-4]
 fid1 = fopen( 'plot_surf_zlogd.in', 'w' );

 nzig = 5;
 x0 = xil + 0.6 * ( xir - xil );
 xzig = 0.3 * ( xir - xil ) / nzig;
 fprintf( fid1, '%f %d %d %d %f %f\n', c, L, P, nzig, x0, xzig );
 boxstart = yil;
 boxheight = yir - yil;
 nbox = 1;
 fprintf( fid1, '%f %f %d\n', boxstart, boxheight, nbox );

 iogx = 0;
 iogy = 0;
 ioad = 0;
 iopx = 1;
 iopy = 1;
 iolb = 0;
 ioub = 0;
 fprintf( fid1, '%d %d %d %d %d %d %d\n', iogx, iogy, ioad, iopx, iopy, iolb, ioub );

 fclose( fid1 );
 system( './lint.x < plot_surf_zlogd.in > plot_surf_zlogd.out' );
 
 x = load( 'fort.13' );
 y = load( 'fort.14' );

 % Note splitting files in half works because we have nbox = 1.
 mx = round( size( x, 1 ) );
 mh = round( mx / 2 );
 plot( x( 1 : mh ), y( 1 : mh ), 'k.-' );
 plot( x( mh + 1 : mx  ), y( mh + 1 : mx ), 'k.-' );

 fn2 = sprintf( 'zzlogd/c%dL%dP%dyl%dyr%dnx%dny%d', round( c ), L, P, round( yil ), round( yir ), nx, ny )
 print('-depsc', '-r300', fn2 )

 end

